import gradio as gr


class ColoredCheckboxGroup(gr.HTML):
    def __init__(
        self,
        choices: list[str],
        *,
        value: list[str] | None = None,
        colors: list[str],
        label: str | None = None,
        **kwargs,
    ):
        html_template = """
        <div class="colored-checkbox-container">
            ${label ? `<label class="container-label">${label}</label>` : ''}
            <div class="colored-checkbox-group">
                ${choices.map((choice, i) => `
                    <label class="checkbox-label" data-color-index="${i}">
                        <input type="checkbox" value="${choice}" ${(value || []).includes(choice) ? 'checked' : ''}>
                        ${choice}
                    </label>
                `).join('')}
            </div>
        </div>
        """

        css_template = """
        .colored-checkbox-container {
            border: 1px solid var(--border-color-primary);
            border-radius: var(--radius-lg);
            padding: var(--spacing-lg);
        }
        .container-label { display: block; margin-bottom: var(--spacing-md); }
        .colored-checkbox-group { display: flex; flex-direction: column; gap: 6px; }
        .checkbox-label { display: flex; align-items: center; cursor: pointer; }
        .checkbox-label input { margin-right: 8px; }
        ${choices.map((choice, i) => `.checkbox-label[data-color-index="${i}"] { color: ${colors[i]}; }`).join(' ')}
        """

        js_on_load = """
        const checkboxes = element.querySelectorAll('input[type="checkbox"]');
        checkboxes.forEach(checkbox => {
            checkbox.addEventListener('change', () => {
                props.value = Array.from(checkboxes)
                    .filter(cb => cb.checked)
                    .map(cb => cb.value);
            });
        });
        """

        super().__init__(
            value=value or [],
            html_template=html_template,
            css_template=css_template,
            js_on_load=js_on_load,
            choices=choices,
            colors=colors,
            label=label,
            **kwargs,
        )

    def api_info(self):
        return {
            "items": {"enum": self.props["choices"], "type": "string"},  # type: ignore
            "title": "Checkbox Group",
            "type": "array",
        }


if __name__ == "__main__":

    def update_colors(color: str):
        if color.startswith("rgb"):
            rgb_values = (
                color.replace("rgba", "").replace("rgb", "").strip("()").split(",")
            )
            r, g, b = (
                int(float(rgb_values[0])),
                int(float(rgb_values[1])),
                int(float(rgb_values[2])),
            )
            medium = f"#{r:02x}{g:02x}{b:02x}"
        else:
            r = int(color[1:3], 16)
            g = int(color[3:5], 16)
            b = int(color[5:7], 16)
            medium = color

        dark = f"#{int(r * 0.6):02x}{int(g * 0.6):02x}{int(b * 0.6):02x}"
        light = f"#{int(r + (255 - r) * 0.4):02x}{int(g + (255 - g) * 0.4):02x}{int(b + (255 - b) * 0.4):02x}"

        return ColoredCheckboxGroup(
            choices=["a", "b", "c"],
            colors=[dark, medium, light],
            label="Select options",
        )

    with gr.Blocks() as demo:
        with gr.Row():
            with gr.Column():
                cp = gr.ColorPicker(value="#FF0000")
            with gr.Column(scale=2):
                cg = ColoredCheckboxGroup(
                    choices=["a", "b", "c"],
                    colors=["#990000", "#FF0000", "#FF6666"],
                    label="Select options",
                )
                gr.Interface(
                    fn=lambda x: " ".join(x),
                    inputs=cg,
                    outputs=gr.Textbox(label="output"),
                )
        cp.change(update_colors, inputs=cp, outputs=cg, show_progress="hidden")
    demo.launch()



from __future__ import annotations

from collections.abc import Iterable

from gradio.themes.base import Base
from gradio.themes.utils import colors, fonts, sizes


class Cyberpunk(Base):
    def __init__(
        self,
        *,
        primary_hue: colors.Color | str = colors.fuchsia,
        secondary_hue: colors.Color | str = colors.cyan,
        neutral_hue: colors.Color | str = colors.slate,
        spacing_size: sizes.Size | str = sizes.spacing_md,
        radius_size: sizes.Size | str = sizes.radius_sm,
        text_size: sizes.Size | str = sizes.text_md,
        font: fonts.Font | str | Iterable[fonts.Font | str] = (
            fonts.GoogleFont("Chakra Petch"),
            "ui-sans-serif",
            "system-ui",
            "sans-serif",
        ),
        font_mono: fonts.Font | str | Iterable[fonts.Font | str] = (
            fonts.GoogleFont("Share Tech Mono"),
            "ui-monospace",
            "Consolas",
            "monospace",
        ),
    ):
        super().__init__(
            primary_hue=primary_hue,
            secondary_hue=secondary_hue,
            neutral_hue=neutral_hue,
            spacing_size=spacing_size,
            radius_size=radius_size,
            text_size=text_size,
            font=font,
            font_mono=font_mono,
        )
        self.name = "cyberpunk"
        super().set(
            # Body — deep violet dark
            body_background_fill="*neutral_100",
            body_background_fill_dark="#0a0416",
            body_text_color="*neutral_800",
            body_text_color_dark="*neutral_300",
            body_text_color_subdued="*neutral_500",
            body_text_color_subdued_dark="*neutral_400",
            # Backgrounds — purple-tinted darks
            background_fill_primary="*neutral_50",
            background_fill_primary_dark="#110b22",
            background_fill_secondary="*neutral_200",
            background_fill_secondary_dark="#1a1030",
            # Shadows — neon bleed only
            shadow_drop="none",
            shadow_drop_lg="none",
            shadow_spread="0px",
            # Blocks — hard-edged, violet-tinted dark panels
            block_background_fill="white",
            block_background_fill_dark="#110b22",
            block_border_width="2px",
            block_border_color="*neutral_300",
            block_border_color_dark=f"{colors.violet.c800}60",
            block_shadow="none",
            block_shadow_dark="none",
            # Labels — neon fuchsia badges
            block_label_background_fill="*primary_100",
            block_label_background_fill_dark=f"{colors.fuchsia.c900}80",
            block_label_text_color="*primary_700",
            block_label_text_color_dark="*primary_300",
            block_label_text_weight="700",
            block_label_text_size="*text_sm",
            block_label_padding="*spacing_sm *spacing_lg",
            block_label_radius="0",
            block_title_background_fill="*primary_100",
            block_title_background_fill_dark=f"{colors.fuchsia.c900}80",
            block_title_text_color="*primary_700",
            block_title_text_color_dark="*primary_300",
            block_title_text_weight="700",
            block_title_padding="*spacing_sm *spacing_lg",
            block_title_radius="0",
            # Inputs — violet-tinted dark, fuchsia focus glow
            input_background_fill="white",
            input_background_fill_dark="#1a1030",
            input_border_color="*neutral_300",
            input_border_color_dark=f"{colors.violet.c700}50",
            input_border_width="2px",
            input_shadow="none",
            input_shadow_focus=f"0 0 0 2px {colors.fuchsia.c300}",
            input_shadow_focus_dark=f"0 0 0 2px {colors.fuchsia.c500}80, 0 0 16px -2px {colors.fuchsia.c600}30",
            input_border_color_focus="*primary_500",
            input_border_color_focus_dark="*primary_500",
            input_background_fill_focus="white",
            input_background_fill_focus_dark="#1e1238",
            # Buttons — solid neon, fuchsia glow in dark
            button_border_width="2px",
            button_transition="all 0.1s ease",
            button_primary_background_fill="*primary_500",
            button_primary_background_fill_hover="*primary_600",
            button_primary_background_fill_dark="*primary_600",
            button_primary_background_fill_hover_dark="*primary_500",
            button_primary_text_color="white",
            button_primary_text_color_dark="white",
            button_primary_border_color="*primary_600",
            button_primary_border_color_dark="*primary_500",
            button_primary_shadow="none",
            button_primary_shadow_hover="none",
            button_primary_shadow_active="none",
            button_primary_shadow_dark=f"0 0 12px -2px {colors.fuchsia.c500}50",
            button_primary_shadow_hover_dark=f"0 0 20px -2px {colors.fuchsia.c500}60",
            button_primary_shadow_active_dark="none",
            button_secondary_background_fill="white",
            button_secondary_background_fill_dark="#1a1030",
            button_secondary_background_fill_hover="*neutral_100",
            button_secondary_background_fill_hover_dark="#241540",
            button_secondary_text_color="*neutral_700",
            button_secondary_text_color_dark="*neutral_300",
            button_secondary_border_color="*neutral_300",
            button_secondary_border_color_dark=f"{colors.violet.c700}50",
            button_secondary_shadow="none",
            button_cancel_background_fill="white",
            button_cancel_background_fill_dark="#1a1030",
            button_cancel_background_fill_hover=colors.red.c50,
            button_cancel_background_fill_hover_dark=f"{colors.red.c900}40",
            button_cancel_text_color=colors.red.c600,
            button_cancel_text_color_dark=colors.red.c400,
            button_cancel_border_color=colors.red.c300,
            button_cancel_border_color_dark=f"{colors.red.c700}60",
            # Checkboxes — fuchsia fill
            checkbox_background_color_selected="*primary_500",
            checkbox_background_color_selected_dark="*primary_500",
            checkbox_border_color="*neutral_400",
            checkbox_border_color_dark=f"{colors.violet.c600}60",
            checkbox_border_color_selected="*primary_600",
            checkbox_border_color_selected_dark="*primary_500",
            checkbox_border_width="2px",
            checkbox_shadow="none",
            checkbox_label_background_fill="white",
            checkbox_label_background_fill_dark="#1a1030",
            checkbox_label_background_fill_hover="*primary_50",
            checkbox_label_background_fill_hover_dark="#241540",
            checkbox_label_background_fill_selected="*primary_600",
            checkbox_label_background_fill_selected_dark="*primary_800",
            checkbox_label_text_color_selected="white",
            checkbox_label_text_color_selected_dark="*primary_200",
            checkbox_label_shadow="none",
            checkbox_label_border_width="2px",
            checkbox_label_border_color="*neutral_300",
            checkbox_label_border_color_dark=f"{colors.violet.c700}50",
            checkbox_label_border_color_selected="*primary_600",
            checkbox_label_border_color_selected_dark="*primary_600",
            # Slider — cyan track
            slider_color="*secondary_500",
            slider_color_dark="*secondary_400",
            # Accents — cyan for selection highlights
            color_accent="*secondary_500",
            color_accent_soft="*secondary_50",
            color_accent_soft_dark=f"{colors.cyan.c900}60",
            # Tables — violet-tinted stripes in dark
            table_even_background_fill="*neutral_50",
            table_even_background_fill_dark="#1a1030",
            table_odd_background_fill="white",
            table_odd_background_fill_dark="#110b22",
            table_border_color="*neutral_300",
            table_border_color_dark=f"{colors.violet.c800}60",
            table_row_focus="*primary_50",
            table_row_focus_dark="#241540",
            # Panels
            panel_border_width="2px",
            panel_border_color="*neutral_300",
            panel_border_color_dark=f"{colors.violet.c800}60",
            # Loader — cyan
            loader_color="*secondary_500",
            loader_color_dark="*secondary_400",
        )



import sys

import pytest

from gradio import Interface, networking

pytestmark = pytest.mark.skipif(
    sys.platform == "win32",
    reason="Skipped on Windows as Windows CI doesn't support tunneling",
)


@pytest.mark.flaky
def test_setup_tunnel():
    io = Interface(lambda x: x, "number", "number")
    io.launch(show_error=True, prevent_thread_lock=True)
    share_url = networking.setup_tunnel(
        io.server_name, io.server_port, io.share_token, io.share_server_address, None
    )
    assert isinstance(share_url, str)



import pytest
from pydantic import BaseModel

import gradio as gr


class TestModel(BaseModel):
    name: str


class TestState:
    def test_as_component(self):
        state = gr.State(value=5)
        assert state.preprocess(10) == 10
        assert state.preprocess("abc") == "abc"
        assert state.stateful

    def test_initial_value_deepcopy(self):
        with pytest.raises(TypeError):
            gr.State(value=gr)  # modules are not deepcopyable

    def test_initial_value_pydantic(self):
        state = gr.State(value=TestModel(name="Freddy"))
        assert isinstance(state.value, TestModel)

    @pytest.mark.asyncio
    async def test_in_interface(self):
        def test(x, y=" def"):
            return (x + y, x + y)

        io = gr.Interface(test, ["text", "state"], ["text", "state"])
        result = await io.call_function(0, ["abc"])
        assert result["prediction"][0] == "abc def"
        result = await io.call_function(0, ["abc", result["prediction"][0]])
        assert result["prediction"][0] == "abcabc def"

    @pytest.mark.asyncio
    async def test_in_blocks(self):
        with gr.Blocks() as demo:
            score = gr.State()
            btn = gr.Button()
            btn.click(lambda x: x + 1, score, score)

        result = await demo.call_function(0, [0])
        assert result["prediction"] == 1
        result = await demo.call_function(0, [result["prediction"]])
        assert result["prediction"] == 2

    @pytest.mark.asyncio
    async def test_update_with_gr_update(self):
        """Test that gr.update(value=...) works with gr.State components."""
        with gr.Blocks() as demo:
            state = gr.State(value=False)
            btn = gr.Button()
            # First function sets state to True using gr.update
            btn.click(lambda: gr.update(value=True), None, state)
            # Second function returns the current state value
            btn2 = gr.Button()
            btn2.click(lambda x: x, state, state)

        # Call first function to set state to True
        result = await demo.call_function(0, [])
        # The prediction is the raw return value (gr.update dict)
        assert result["prediction"] == {"__type__": "update", "value": True}

        # Call second function to verify state was actually updated
        result = await demo.call_function(1, [True])  # Pass True as current state
        assert result["prediction"] is True

    @pytest.mark.asyncio
    async def test_update_with_gr_update_complex_value(self):
        """Test that gr.update(value=...) works with complex values."""
        with gr.Blocks() as demo:
            state = gr.State(value={"count": 0})
            btn = gr.Button()
            btn.click(
                lambda x: gr.update(value={"count": x["count"] + 1}), state, state
            )

        result = await demo.call_function(0, [{"count": 5}])
        # The prediction is the raw return value (gr.update dict)
        assert result["prediction"] == {"__type__": "update", "value": {"count": 6}}



"""gr.DateTime() component."""

from __future__ import annotations

import re
from datetime import datetime, timedelta
from typing import Any, Literal

import pytz
from gradio_client.documentation import document

from gradio.components.base import FormComponent
from gradio.components.button import Button
from gradio.events import Events
from gradio.i18n import I18nData
from gradio.utils import set_default_buttons


@document()
class DateTime(FormComponent):
    """
    Component to select a date and (optionally) a time.

    Guides: time-plots
    """

    EVENTS = [
        Events.change,
        Events.submit,
    ]

    def __init__(
        self,
        value: float | str | datetime | None = None,
        *,
        include_time: bool = True,
        type: Literal["timestamp", "datetime", "string"] = "timestamp",
        timezone: str | None = None,
        label: str | I18nData | None = None,
        show_label: bool | None = None,
        info: str | I18nData | None = None,
        every: float | None = None,
        scale: int | None = None,
        min_width: int = 160,
        visible: bool | Literal["hidden"] = True,
        interactive: bool | None = None,
        elem_id: str | None = None,
        elem_classes: list[str] | str | None = None,
        render: bool = True,
        key: int | str | tuple[int | str, ...] | None = None,
        preserved_by_key: list[str] | str | None = "value",
        buttons: list[Button] | None = None,
    ):
        """
        Parameters:
            value: default value for datetime.
            label: the label for this component, displayed above the component if `show_label` is `True` and is also used as the header if there are a table of examples for this component. If None and used in a `gr.Interface`, the label will be the name of the parameter this component corresponds to.
            info: additional component description, appears below the label in smaller font. Supports markdown / HTML syntax.
            show_label: if True, will display label.
            include_time: If True, the component will include time selection. If False, only date selection will be available.
            type: The type of the value. Can be "timestamp", "datetime", or "string". If "timestamp", the value will be a number representing the start and end date in seconds since epoch. If "datetime", the value will be a datetime object. If "string", the value will be the date entered by the user.
            timezone: The timezone to use for timestamps, such as "US/Pacific" or "Europe/Paris". If None, the timezone will be the local timezone.
            every: If `value` is a callable, run the function 'every' number of seconds while the client connection is open. Has no effect otherwise. The event can be accessed (e.g. to cancel it) via this component's .load_event attribute.
            scale: relative size compared to adjacent Components. For example if Components A and B are in a Row, and A has scale=2, and B has scale=1, A will be twice as wide as B. Should be an integer. scale applies in Rows, and to top-level Components in Blocks where fill_height=True.
            min_width: minimum pixel width, will wrap if not sufficient screen space to satisfy this value. If a certain scale value results in this Component being narrower than min_width, the min_width parameter will be respected first.
            visible: If False, component will be hidden. If "hidden", component will be visually hidden and not take up space in the layout but still exist in the DOM
            elem_classes: An optional list of strings that are assigned as the classes of this component in the HTML DOM. Can be used for targeting CSS styles.
            render: If False, component will not render be rendered in the Blocks context. Should be used if the intention is to assign event listeners now but render the component later.
            key: in a gr.render, Components with the same key across re-renders are treated as the same component, not a new component. Properties set in 'preserved_by_key' are not reset across a re-render.
            preserved_by_key: A list of parameters from this component's constructor. Inside a gr.render() function, if a component is re-rendered with the same key, these (and only these) parameters will be preserved in the UI (if they have been changed by the user or an event listener) instead of re-rendered based on the values provided during constructor.
            buttons: A list of gr.Button() instances to show in the top right corner of the component. Custom buttons will appear in the toolbar with their configured icon and/or label, and clicking them will trigger any .click() events registered on the button.
        """
        self.type = type
        self.include_time = include_time
        self.time_format = "%Y-%m-%d %H:%M:%S" if include_time else "%Y-%m-%d"
        self.timezone = timezone
        self._value_description = (
            "a datetime object"
            if self.type == "datetime"
            else "a unix timestamp"
            if self.type == "timestamp"
            else "a %Y-%m-%d %H:%M:%S formatted string"
            if include_time
            else "a %Y-%m-%d formatted string"
        )
        super().__init__(
            every=every,
            scale=scale,
            min_width=min_width,
            visible=visible,
            label=label,
            show_label=show_label,
            info=info,
            elem_id=elem_id,
            elem_classes=elem_classes,
            render=render,
            key=key,
            preserved_by_key=preserved_by_key,
            value=value,
            interactive=interactive,
        )
        self.buttons = set_default_buttons(buttons, None)

    def preprocess(self, payload: str | None) -> str | float | datetime | None:
        """
        Parameters:
            payload: the text entered in the textarea.
        Returns:
            Passes text value as a {str} into the function.
        """
        if payload is None or payload == "":
            return None
        if self.type == "string" and "now" not in payload:
            return payload
        datetime = self.get_datetime_from_str(payload)
        if self.type == "string":
            return datetime.strftime(self.time_format)
        if self.type == "datetime":
            return datetime
        elif self.type == "timestamp":
            return datetime.timestamp()

    def postprocess(self, value: float | datetime | str | None) -> str | None:
        """
        Parameters:
            value: Expects a tuple pair of datetimes.
        Returns:
            A tuple pair of timestamps.
        """
        if value is None:
            return None

        if isinstance(value, datetime):
            return datetime.strftime(value, self.time_format)
        elif isinstance(value, str):
            return value
        else:
            return datetime.fromtimestamp(
                value, tz=pytz.timezone(self.timezone) if self.timezone else None
            ).strftime(self.time_format)

    def api_info(self) -> dict[str, Any]:
        return {
            "type": "string",
            "description": f"Formatted as YYYY-MM-DD{' HH:MM:SS' if self.include_time else ''}",
        }

    def example_payload(self) -> str:
        return "2020-10-01 05:20:15"

    def example_value(self) -> str:
        return "2020-10-01 05:20:15"

    def get_datetime_from_str(self, date: str) -> datetime:
        now_regex = r"^(?:\s*now\s*(?:-\s*(\d+)\s*([dmhs]))?)?\s*$"

        if "now" in date:
            match = re.match(now_regex, date)
            if match:
                num = int(match.group(1) or 0)
                unit = match.group(2) or "s"
                if unit == "d":
                    delta = timedelta(days=num)
                elif unit == "h":
                    delta = timedelta(hours=num)
                elif unit == "m":
                    delta = timedelta(minutes=num)
                else:
                    delta = timedelta(seconds=num)
                return datetime.now() - delta
            else:
                raise ValueError("Invalid 'now' time format")
        else:
            dt = datetime.strptime(date, self.time_format)
            if self.timezone:
                dt = pytz.timezone(self.timezone).localize(dt)
            return dt



from __future__ import annotations

from gradio_client.documentation import document
from gradio_client.exceptions import AppError


class DuplicateBlockError(ValueError):
    """Raised when a Blocks contains more than one Block with the same id"""

    pass


class InvalidComponentError(ValueError):
    """Raised when invalid components are used."""

    pass


class ModelNotFoundError(Exception):
    """Raised when the provided model doesn't exists or is not found by the provided api url."""

    pass


class RenderError(Exception):
    """Raised when a component has not been rendered in the current Blocks but is expected to have been rendered."""

    pass


class InvalidApiNameError(ValueError):
    pass


class ServerFailedToStartError(Exception):
    pass


class InvalidBlockError(ValueError):
    """Raised when an event in a Blocks contains a reference to a Block that is not in the original Blocks"""

    pass


class ReloadError(ValueError):
    """Raised when something goes wrong when reloading the gradio app."""

    pass


class GradioVersionIncompatibleError(Exception):
    """Raised when loading a 3.x space with 4.0"""

    pass


InvalidApiName = InvalidApiNameError  # backwards compatibility


@document(documentation_group="modals")
class Error(AppError):
    """
    This class allows you to pass custom error messages to the user. You can do so by raising a gr.Error("custom message") anywhere in the code, and when that line is executed the custom message will appear in a modal on the demo.
    Example:
        import gradio as gr
        def divide(numerator, denominator):
            if denominator == 0:
                raise gr.Error("Cannot divide by zero!")
        gr.Interface(divide, ["number", "number"], "number").launch()
    Demos: calculator, blocks_chained_events
    Guides: alerts
    """

    def __init__(
        self,
        message: str = "Error raised.",
        duration: float | None = 10,
        visible: bool = True,
        title: str = "Error",
        print_exception: bool = True,
    ):
        """
        Parameters:
            message: The error message to be displayed to the user. Can be HTML, which will be rendered in the modal.
            duration: The duration in seconds to display the error message. If None or 0, the error message will be displayed until the user closes it.
            visible: Whether the error message should be displayed in the UI.
            title: The title to be displayed to the user at the top of the error modal.
            print_exception: Whether to print traceback of the error to the console when the error is raised.
        """
        super().__init__(
            message=message,
            duration=duration,
            visible=visible,
            title=title,
            print_exception=print_exception,
        )

    def __str__(self):
        return repr(self.message)


class ComponentDefinitionError(NotImplementedError):
    pass


class InvalidPathError(ValueError):
    pass


class ChecksumMismatchError(Exception):
    pass


class TooManyRequestsError(Error):
    """Raised when the Hugging Face API returns a 429 status code."""

    def __init__(self, message: str = "Too many requests. Please try again later."):
        super().__init__(message)



import asyncio
import copy
import io
import json
import os
import pathlib
import random
import sys
import threading
import time
import uuid
import warnings
from concurrent.futures import wait
from contextlib import contextmanager
from functools import partial
from string import capwords
from unittest.mock import mock_open, patch

import gradio_client as grc
import numpy as np
import pytest
from fastapi import FastAPI
from fastapi.testclient import TestClient
from gradio_client import Client
from PIL import Image

import gradio as gr
from gradio import blocks, helpers
from gradio.data_classes import GradioModel, GradioRootModel
from gradio.events import SelectData
from gradio.exceptions import DuplicateBlockError
from gradio.route_utils import API_PREFIX
from gradio.utils import assert_configs_are_equivalent_besides_ids, cancel_tasks

pytest_plugins = ("pytest_asyncio",)

os.environ["GRADIO_ANALYTICS_ENABLED"] = "False"


@contextmanager
def captured_output():
    new_out, new_err = io.StringIO(), io.StringIO()
    old_out, old_err = sys.stdout, sys.stderr
    try:
        sys.stdout, sys.stderr = new_out, new_err
        yield sys.stdout, sys.stderr
    finally:
        sys.stdout, sys.stderr = old_out, old_err


class TestBlocksMethods:
    maxDiff = None

    def test_set_share_is_false_by_default(self):
        with gr.Blocks() as demo:
            assert not demo.share

    @patch("gradio.networking.setup_tunnel")
    @patch("gradio.utils.colab_check")
    def test_set_share_in_colab(self, mock_colab_check, mock_setup_tunnel):
        mock_colab_check.return_value = True
        mock_setup_tunnel.return_value = "http://localhost:7860/"
        with gr.Blocks() as demo:
            # self.share is False when instantiating the class
            assert not demo.share
            # share default is True, if share is None in colab and queueing
            demo.launch(prevent_thread_lock=True)
            assert demo.share
            demo.close()
            # share is also true, if share is None in colab with queueing
            demo.queue()
            demo.launch(prevent_thread_lock=True)
            assert demo.share
            demo.close()

    def test_load_from_config(self):
        fake_url = "https://fake.hf.space"

        def update(name):
            return f"Welcome to Gradio, {name}!"

        with gr.Blocks() as demo1:
            inp = gr.Textbox(placeholder="What is your name?")
            out = gr.Textbox()

            inp.submit(fn=update, inputs=inp, outputs=out, api_name="greet")

            gr.Image(height=54, width=240)

        config1 = demo1.get_config_file()
        demo2 = gr.Blocks.from_config(config1, [update], "https://fake.hf.space")

        for component in config1["components"]:
            component["props"]["proxy_url"] = f"{fake_url}/"
        config2 = demo2.get_config_file()
        assert assert_configs_are_equivalent_besides_ids(config1, config2)

    def test_load_from_config_with_blocks_events(self):
        fake_url = "https://fake.hf.space"

        def fn():
            return "Hello"

        with gr.Blocks() as demo:
            t = gr.Textbox()
            demo.load(fn, None, t)

        config = demo.get_config_file()
        gr.Blocks.from_config(config, [fn], fake_url)  # Should not raise

    def test_partial_fn_in_config(self):
        def greet(name, formatter):
            return formatter(f"Hello {name}!")

        greet_upper_case = partial(greet, formatter=capwords)
        with gr.Blocks() as demo:
            t = gr.Textbox()
            o = gr.Textbox()
            t.change(greet_upper_case, t, o)

        assert len(demo.fns) == 1
        assert "fn" in str(demo.fns[0])

    @pytest.mark.asyncio
    async def test_dict_inputs_in_config(self):
        with gr.Blocks() as demo:
            first = gr.Textbox()
            last = gr.Textbox()
            btn = gr.Button()
            greeting = gr.Textbox()

            def greet(data):
                return f"Hello {data[first]} {data[last]}"

            btn.click(greet, {first, last}, greeting)

        result = await demo.process_api(
            inputs=["huggy", "face"], block_fn=0, state=None
        )
        assert result["data"] == ["Hello huggy face"]

    @pytest.mark.asyncio
    async def test_async_function(self):
        async def wait(x):
            await asyncio.sleep(0.01)
            return x

        with gr.Blocks() as demo:
            text = gr.Textbox()
            button = gr.Button()
            button.click(wait, [text], [text])

            start = time.time()
            result = await demo.process_api(inputs=[1], block_fn=0, state=None)
            end = time.time()
            difference = end - start
            assert difference >= 0.01
            assert result

    @patch("gradio.analytics._do_analytics_request")
    def test_initiated_analytics(self, mock_anlaytics, monkeypatch):
        monkeypatch.setenv("GRADIO_ANALYTICS_ENABLED", "True")
        with gr.Blocks():
            pass
        mock_anlaytics.assert_called_once()

    @patch("gradio.analytics._do_analytics_request")
    def test_launch_analytics_does_not_error_with_invalid_blocks(
        self, mock_anlaytics, monkeypatch
    ):
        monkeypatch.setenv("GRADIO_ANALYTICS_ENABLED", "True")
        with gr.Blocks():
            t1 = gr.Textbox()

        with gr.Blocks() as demo:
            t2 = gr.Textbox()
            t2.change(lambda x: x, t2, t1)

        demo.launch(prevent_thread_lock=True)
        mock_anlaytics.assert_called()

    def test_show_error(self):
        with gr.Blocks() as demo:
            pass

        assert demo.show_error
        demo.launch(prevent_thread_lock=True)
        assert not demo.show_error
        demo.close()
        demo.launch(show_error=True, prevent_thread_lock=True)
        assert demo.show_error
        demo.close()

    def test_custom_css(self):
        css = """
            .gr-button {
                color: white;
                border-color: black;
                background: black;
            }
        """
        css = css * 5  # simulate a long css string
        with gr.Blocks() as block:
            pass
        block.launch(prevent_thread_lock=True, css=css)
        assert block.css == css
        block.close()

    @pytest.mark.asyncio
    async def test_restart_after_close(self, connect):
        io = gr.Interface(
            lambda s: s, gr.Textbox(), gr.Textbox(), api_name="predict"
        ).queue()

        with connect(io) as client:
            assert client.predict("freddy", api_name="/predict") == "freddy"
        # connect launches the interface which is what we need to test
        with connect(io) as client:
            assert client.predict("Victor", api_name="/predict") == "Victor"

    @pytest.mark.asyncio
    async def test_async_generators(self, connect):
        async def async_iteration(count: int):
            for i in range(count):
                yield i
                await asyncio.sleep(0.2)

        def iteration(count: int):
            for i in range(count):
                yield i
                time.sleep(0.2)

        with gr.Blocks() as demo:
            with gr.Row():
                with gr.Column():
                    num1 = gr.Number(value=4, precision=0)
                    o1 = gr.Number()
                    async_iterate = gr.Button(value="Async Iteration")
                    async_iterate.click(
                        async_iteration,
                        num1,
                        o1,
                        concurrency_limit=2,
                        concurrency_id="main",
                    )
                with gr.Column():
                    num2 = gr.Number(value=4, precision=0)
                    o2 = gr.Number()
                    iterate = gr.Button(value="Iterate")
                    iterate.click(iteration, num2, o2, concurrency_id="main")

        with connect(demo) as client:
            job_1 = client.submit(3, fn_index=0)
            job_2 = client.submit(4, fn_index=1)
            wait([job_1, job_2])

            assert job_1.outputs()[-1] == 2
            assert job_2.outputs()[-1] == 3

    def test_async_generators_interface(self, connect):
        async def async_iteration(count: int):
            for i in range(count):
                yield i
                await asyncio.sleep(0.2)

        demo = gr.Interface(
            async_iteration, gr.Number(precision=0), gr.Number(), api_name="predict"
        ).queue()
        outputs = []
        with connect(demo) as client:
            for output in client.submit(3, api_name="/predict"):
                outputs.append(output)
        assert outputs == [0, 1, 2]

    def test_sync_generators(self, connect):
        def generator(string):
            yield from string

        demo = gr.Interface(generator, "text", "text", api_name="predict").queue()
        outputs = []
        with connect(demo) as client:
            for output in client.submit("abc", api_name="/predict"):
                outputs.append(output)
        assert outputs == ["a", "b", "c"]
        demo.queue().launch(prevent_thread_lock=True)

    def test_varying_output_forms_with_generators(self, connect):
        generations = [
            {"a": 1},
            {"a": 1, "b": [1, 3]},
            {"b": [1, 3, 2]},
            1,
            2,
            3,
            [1, 2, {"x": 4, "y": 6}],
            {"data": [1, 2, {"x": 4, "y": 6}]},
            None,
            1.2,
        ]

        def generator():
            yield from generations

        def generator_random():
            indices = list(range(len(generations)))
            random.shuffle(indices)
            for i in indices:
                time.sleep(random.random() / 5)
                yield generations[i]

        with gr.Blocks() as demo:
            btn1 = gr.Button()
            btn2 = gr.Button()
            output_json = gr.JSON()
            btn1.click(generator, None, output_json, api_name="generator")
            btn2.click(generator_random, None, output_json, api_name="generator_random")

        with connect(demo) as client:
            outputs = []
            for output in client.submit(api_name="/generator"):
                outputs.append(output)
            assert outputs == generations

            outputs = []
            for output in client.submit(api_name="/generator_random"):
                outputs.append(output)
            for generation in generations:
                assert generation in outputs

    def test_socket_reuse(self):
        try:
            io = gr.Interface(lambda x: x, gr.Textbox(), gr.Textbox())
            io.launch(server_port=9441, prevent_thread_lock=True)
            io.close()
            io.launch(server_port=9441, prevent_thread_lock=True)
        finally:
            io.close()  # type: ignore

    def test_function_types_documented_in_config(self):
        def continuous_fn():
            return 42

        def generator_function():
            yield from range(10)

        with gr.Blocks() as demo:
            gr.Number(value=lambda: 2, every=2)
            meaning_of_life = gr.Number()
            counter = gr.Number()
            generator_btn = gr.Button(value="Generate")
            greeting = gr.Textbox()
            greet_btn = gr.Button(value="Greet")

            greet_btn.click(lambda: "Hello!", inputs=None, outputs=[greeting])
            generator_btn.click(generator_function, inputs=None, outputs=[counter])
            demo.load(continuous_fn, inputs=None, outputs=[meaning_of_life])

        assert "dependencies" in demo.config
        dependencies = demo.config["dependencies"]
        assert dependencies[0]["types"] == {
            "generator": False,
            "cancel": False,
        }
        assert dependencies[1]["types"] == {
            "generator": True,
            "cancel": False,
        }
        assert dependencies[2]["types"] == {
            "generator": False,
            "cancel": False,
        }
        assert dependencies[3]["types"] == {
            "generator": False,
            "cancel": False,
        }

    @patch(
        "gradio.themes.ThemeClass.from_hub",
        side_effect=ValueError("Something went wrong!"),
    )
    def test_use_default_theme_as_fallback(self, mock_from_hub):
        with pytest.warns(
            UserWarning, match="Cannot load freddyaboulton/this-theme-does-not-exist"
        ):
            with gr.Blocks() as demo:
                pass
            demo.launch(
                prevent_thread_lock=True,
                theme="freddyaboulton/this-theme-does-not-exist",
            )
            assert demo.theme.to_dict() == gr.themes.Default().to_dict()
            demo.close()

    def test_exit_called_at_launch(self):
        with gr.Blocks() as demo:
            gr.Textbox(uuid.uuid4)
        demo.launch(prevent_thread_lock=True)
        config = demo.get_config_file()
        assert "dependencies" in config
        assert len(config["dependencies"]) == 1


class TestTempFile:
    def test_pil_images_hashed(self, connect, gradio_temp_dir):
        images = [
            Image.new("RGB", (512, 512), color) for color in ("red", "green", "blue")
        ]

        def create_images(n_images):
            return random.sample(images, n_images)

        gallery = gr.Gallery()
        demo = gr.Interface(
            create_images,
            inputs="slider",
            outputs=gallery,
            api_name="predict",
        )
        with connect(demo) as client:
            client.predict(3, api_name="/predict")
            _ = client.predict(3, api_name="/predict")
        # only three files created and in temp directory
        assert len([f for f in gradio_temp_dir.glob("**/*") if f.is_file()]) == 3

    def test_no_empty_image_files(self, gradio_temp_dir, connect):
        file_dir = pathlib.Path(__file__).parent / "test_files"
        image = grc.handle_file(str(file_dir / "bus.png"))

        demo = gr.Interface(
            lambda x: x,
            inputs=gr.Image(type="filepath"),
            outputs=gr.Image(),
            api_name="predict",
        )
        with connect(demo) as client:
            _ = client.predict(image, api_name="/predict")
            _ = client.predict(image, api_name="/predict")
            _ = client.predict(image, api_name="/predict")
        # Upload creates a file. image preprocessing creates another one.
        assert len([f for f in gradio_temp_dir.glob("**/*") if f.is_file()]) == 2

    @pytest.mark.parametrize("component", [gr.UploadButton, gr.File])
    def test_file_component_uploads(self, component, connect, gradio_temp_dir):
        code_file = grc.handle_file(str(pathlib.Path(__file__)))
        demo = gr.Interface(
            lambda x: x.name, component(), gr.File(), api_name="predict"
        )
        with connect(demo) as client:
            _ = client.predict(code_file, api_name="/predict")
            _ = client.predict(code_file, api_name="/predict")
        # the upload route hashees the files so we get 1 from there
        # We create two tempfiles (empty) because API says we return
        # preprocess/postprocess will create the same file as the upload route
        # so 1 + 2 = 3
        assert len([f for f in gradio_temp_dir.glob("**/*") if f.is_file()]) == 3

    def test_no_empty_video_files(self, gradio_temp_dir, connect):
        file_dir = pathlib.Path(pathlib.Path(__file__).parent, "test_files")
        video = grc.handle_file(str(file_dir / "video_sample.mp4"))
        demo = gr.Interface(lambda x: x, gr.Video(), gr.Video(), api_name="predict")
        with connect(demo) as client:
            _ = client.predict(video, api_name="/predict")
            _ = client.predict(video, api_name="/predict")
        # Upload route and postprocessing return the same file
        assert len([f for f in gradio_temp_dir.glob("**/*") if f.is_file()]) == 1

    def test_no_empty_audio_files(self, gradio_temp_dir, connect):
        file_dir = pathlib.Path(pathlib.Path(__file__).parent, "test_files")
        audio = grc.handle_file(str(file_dir / "audio_sample.wav"))

        def reverse_audio(audio):
            sr, data = audio
            return (sr, np.flipud(data))

        demo = gr.Interface(
            fn=reverse_audio, inputs=gr.Audio(), outputs=gr.Audio(), api_name="predict"
        )
        with connect(demo) as client:
            _ = client.predict(audio, api_name="/predict")
            _ = client.predict(audio, api_name="/predict")
            # One for upload and one for reversal
            assert len([f for f in gradio_temp_dir.glob("**/*") if f.is_file()]) == 2


class TestComponentsInBlocks:
    def test_slider_random_value_config(self):
        with gr.Blocks() as demo:
            gr.Slider(
                value=11.2,
                minimum=-10.2,
                maximum=15,
                label="Non-random Slider (Static)",
            )
            gr.Slider(
                randomize=True,
                minimum=100,
                maximum=200,
                label="Random Slider (Input 1)",
            )
            gr.Slider(
                randomize=True,
                minimum=10,
                maximum=23.2,
                label="Random Slider (Input 2)",
            )
        for component in demo.blocks.values():
            if isinstance(component, gr.components.Component):
                if "Non-random" in component.label:  # type: ignore
                    assert not component.load_event_to_attach
                else:
                    assert component.load_event_to_attach
        assert "dependencies" in demo.config
        dependencies_on_load = [
            dep["targets"][0][1] == "load" for dep in demo.config["dependencies"]
        ]
        assert all(dependencies_on_load)
        assert len(dependencies_on_load) == 2

    def test_io_components_attach_load_events_when_value_is_fn(self, io_components):
        interface = gr.Interface(
            lambda *args: None,
            inputs=[comp(value=lambda: None, every=1) for comp in io_components],
            outputs=None,
        )
        assert "dependencies" in interface.config
        dependencies_on_load = [
            dep
            for dep in interface.config["dependencies"]
            if "load" in [target[1] for target in dep["targets"]]
        ]
        dependencies_on_tick = [
            dep
            for dep in interface.config["dependencies"]
            if "tick" in [target[1] for target in dep["targets"]]
        ]
        assert len(dependencies_on_load) == len(io_components)
        assert len(dependencies_on_tick) == len(io_components)

    def test_get_load_events(self, io_components):
        components = []
        with gr.Blocks() as demo:
            for component in io_components:
                components.append(component(value=lambda: None, every=1))
        assert "dependencies" in demo.config
        assert all(
            comp.load_event in demo.config["dependencies"] for comp in components
        )

    def test_load_events_work_with_builtins(self):
        with gr.Blocks() as demo:
            gr.State(dict)
        demo.get_api_info()


class TestBlocksPostprocessing:
    @pytest.mark.asyncio
    async def test_blocks_do_not_filter_none_values_from_updates(self, io_components):
        io_components = [
            c()
            for c in io_components
            if c
            not in [
                gr.State,
                gr.Button,
                gr.ScatterPlot,
                gr.LinePlot,
                gr.BarPlot,
                gr.components.Fallback,
                gr.FileExplorer,
                gr.ParamViewer,
            ]
        ]
        with gr.Blocks() as demo:
            for component in io_components:
                component.render()
            btn = gr.Button(value="Reset")
            btn.click(
                lambda: [gr.update(value=None) for _ in io_components],
                inputs=[],
                outputs=io_components,
            )

        output = await demo.postprocess_data(
            demo.fns[0], [gr.update(value=None) for _ in io_components], state=None
        )

        def process_and_dump(component):
            output = component.postprocess(None)
            if isinstance(output, (GradioModel, GradioRootModel)):
                output = output.model_dump()
            return output

        assert all(
            o["value"] == process_and_dump(c)
            for o, c in zip(output, io_components, strict=False)
        )

    @pytest.mark.asyncio
    async def test_blocks_does_not_replace_keyword_literal(self):
        with gr.Blocks() as demo:
            text = gr.Textbox()
            btn = gr.Button(value="Reset")
            btn.click(
                lambda: gr.update(value="NO_VALUE"),
                inputs=[],
                outputs=text,
            )

        output = await demo.postprocess_data(
            demo.fns[0], gr.update(value="NO_VALUE"), state=None
        )
        assert output[0]["value"] == "NO_VALUE"

    @pytest.mark.asyncio
    async def test_blocks_does_not_del_dict_keys_inplace(self):
        with gr.Blocks() as demo:
            im_list = [gr.Image() for i in range(2)]

            def change_visibility(value):
                return [gr.update(visible=value)] * 2

            checkbox = gr.Checkbox(value=True, label="Show image")
            checkbox.change(change_visibility, inputs=checkbox, outputs=im_list)

        output = await demo.postprocess_data(
            demo.fns[0], [gr.update(visible=False)] * 2, state=None
        )
        assert output == [
            {"visible": False, "__type__": "update"},
            {"visible": False, "__type__": "update"},
        ]

    @pytest.mark.asyncio
    async def test_blocks_returns_correct_output_dict_single_key(self):
        with gr.Blocks() as demo:
            num = gr.Number()
            num2 = gr.Number()
            update = gr.Button(value="update")

            def update_values(val):
                return {num2: gr.Number(value=42)}

            update.click(update_values, inputs=[num], outputs=[num2])

        output = await demo.postprocess_data(
            demo.fns[0], {num2: gr.Number(value=42)}, state=None
        )
        assert output[0]["value"] == 42

        output = await demo.postprocess_data(demo.fns[0], {num2: 23}, state=None)
        assert output[0] == 23

    @pytest.mark.asyncio
    async def test_blocks_update_dict_without_postprocessing(self, media_data):
        def infer(x):
            return media_data.BASE64_IMAGE, gr.update(visible=True)

        with gr.Blocks() as demo:
            prompt = gr.Textbox()
            image = gr.Image()
            run_button = gr.Button()
            share_button = gr.Button("share", visible=False)
            run_button.click(infer, prompt, [image, share_button], postprocess=False)

        output = await demo.process_api(0, ["test"], state=None)
        assert output["data"][0] == media_data.BASE64_IMAGE
        assert output["data"][1] == {"__type__": "update", "visible": True}

    @pytest.mark.asyncio
    async def test_blocks_update_dict_does_not_postprocess_value_if_postprocessing_false(
        self, media_data
    ):
        def infer(x):
            return gr.Image(value=media_data.BASE64_IMAGE)

        with gr.Blocks() as demo:
            prompt = gr.Textbox()
            image = gr.Image()
            run_button = gr.Button()
            run_button.click(infer, [prompt], [image], postprocess=False)

        output = await demo.process_api(0, ["test"], state=None)
        assert output["data"][0] == {
            "__type__": "update",
            "value": media_data.BASE64_IMAGE,
        }

    @pytest.mark.asyncio
    async def test_blocks_update_interactive(
        self,
    ):
        def specific_update():
            return [
                gr.Image(interactive=True),
                gr.Textbox(interactive=True),
            ]

        def generic_update():
            return [gr.update(interactive=True), gr.update(interactive=True)]

        with gr.Blocks() as demo:
            run = gr.Button(value="Make interactive")
            image = gr.Image()
            textbox = gr.Text()
            run.click(specific_update, None, [image, textbox])
            run.click(generic_update, None, [image, textbox])

        for fn_index in range(2):
            output = await demo.process_api(fn_index, [], state=None)
            assert output["data"][0] == {
                "__type__": "update",
                "interactive": True,
            }
            assert output["data"][1] == {"__type__": "update", "interactive": True}

    @pytest.mark.asyncio
    async def test_error_raised_if_num_outputs_is_too_low(self):
        with gr.Blocks() as demo:
            textbox1 = gr.Textbox()
            textbox2 = gr.Textbox()
            button = gr.Button()
            button.click(lambda x: x, textbox1, [textbox1, textbox2])
        with pytest.raises(
            ValueError,
        ):
            await demo.postprocess_data(demo.fns[0], predictions=["test"], state=None)

    @pytest.mark.asyncio
    async def test_warning_raised_if_num_outputs_is_too_high(self):
        with gr.Blocks() as demo:
            textbox1 = gr.Textbox()
            textbox2 = gr.Textbox()
            button = gr.Button()
            button.click(lambda x: (x, x), textbox1, [textbox1, textbox2])
        with pytest.warns(
            UserWarning,
        ):
            await demo.postprocess_data(
                demo.fns[0], predictions=["test", "test2", "test3"], state=None
            )

    @pytest.mark.asyncio
    async def test_no_warning_if_func_has_no_outputs(self):
        """
        Ensures that if a function has no outputs, no warning is raised.
        """
        with gr.Blocks() as demo:
            button = gr.Button()

            def no_return():
                pass

            button.click(
                no_return,
                inputs=None,
                outputs=None,
            )
        with warnings.catch_warnings():
            warnings.simplefilter("error")
            await demo.postprocess_data(demo.fns[0], predictions=None, state=None)  # type: ignore

    @pytest.mark.asyncio
    async def test_error_raised_if_num_outputs_mismatch_with_function_name(self):
        def infer(x):
            return x

        with gr.Blocks() as demo:
            textbox1 = gr.Textbox()
            textbox2 = gr.Textbox()
            button = gr.Button()
            button.click(infer, textbox1, [textbox1, textbox2])
        with pytest.raises(
            ValueError,
        ):
            await demo.postprocess_data(demo.fns[0], predictions=["test"], state=None)

    @pytest.mark.asyncio
    async def test_error_raised_if_num_outputs_mismatch_single_output(self):
        with gr.Blocks() as demo:
            num1 = gr.Number()
            num2 = gr.Number()
            btn = gr.Button(value="1")
            btn.click(lambda a: a, num1, [num1, num2])
        with pytest.raises(
            ValueError,
        ):
            await demo.postprocess_data(demo.fns[0], predictions=[1], state=None)

    @pytest.mark.asyncio
    async def test_error_raised_if_num_outputs_mismatch_tuple_output(self):
        def infer(a, b):
            return a, b

        with gr.Blocks() as demo:
            num1 = gr.Number()
            num2 = gr.Number()
            num3 = gr.Number()
            btn = gr.Button(value="1")
            btn.click(infer, num1, [num1, num2, num3])
        with pytest.raises(
            ValueError,
        ):
            await demo.postprocess_data(demo.fns[0], predictions=[1, 2], state=None)

    @pytest.mark.asyncio
    async def test_dataset_is_updated(self):
        def update(value):
            return value, gr.Dataset(samples=[["New A"], ["New B"]])

        with gr.Blocks() as demo:
            with gr.Row():
                textbox = gr.Textbox()
                dataset = gr.Dataset(
                    components=["text"], samples=[["Original"]], label="Saved Prompts"
                )
                dataset.click(
                    update,
                    inputs=[dataset],
                    outputs=[textbox, dataset],
                    api_name="predict",
                )
        app, _, _ = demo.launch(prevent_thread_lock=True)

        client = TestClient(app)

        session_1 = client.post(
            f"{API_PREFIX}/api/predict/",
            json={"data": [0], "session_hash": "1", "fn_index": 0},
        )
        assert "Original" in session_1.json()["data"][0]
        session_2 = client.post(
            f"{API_PREFIX}/api/predict/",
            json={"data": [0], "session_hash": "1", "fn_index": 0},
        )
        assert "New" in session_2.json()["data"][0]


class TestStateHolder:
    @pytest.mark.asyncio
    async def test_state_stored_up_to_capacity(self):
        with gr.Blocks() as demo:
            num = gr.Number()
            state = gr.State(value=0)

            def run(x, s):
                return s, s + 1

            num.submit(
                run,
                inputs=[num, state],
                outputs=[num, state],
                api_name="predict",
            )
        app, _, _ = demo.launch(prevent_thread_lock=True, state_session_capacity=2)
        client = TestClient(app)

        session_1 = client.post(
            f"{API_PREFIX}/api/predict/",
            json={"data": [1, None], "session_hash": "1", "fn_index": 0},
        )
        assert session_1.json()["data"][0] == 0
        session_2 = client.post(
            f"{API_PREFIX}/api/predict/",
            json={"data": [1, None], "session_hash": "2", "fn_index": 0},
        )
        assert session_2.json()["data"][0] == 0
        session_1 = client.post(
            f"{API_PREFIX}/api/predict/",
            json={"data": [1, None], "session_hash": "1", "fn_index": 0},
        )
        assert session_1.json()["data"][0] == 1
        session_2 = client.post(
            f"{API_PREFIX}/api/predict/",
            json={"data": [1, None], "session_hash": "2", "fn_index": 0},
        )
        assert session_2.json()["data"][0] == 1
        session_3 = client.post(
            f"{API_PREFIX}/api/predict/",
            json={"data": [1, None], "session_hash": "3", "fn_index": 0},
        )
        assert session_3.json()["data"][0] == 0
        session_2 = client.post(
            f"{API_PREFIX}/api/predict/",
            json={"data": [1, None], "session_hash": "2", "fn_index": 0},
        )
        assert session_2.json()["data"][0] == 2
        session_1 = client.post(
            f"{API_PREFIX}/api/predict/",
            json={"data": [1, None], "session_hash": "1", "fn_index": 0},
        )
        assert (
            session_1.json()["data"][0] == 0
        )  # state was lost for session 1 when session 3 was added, since state_session_capacity=2

    @pytest.mark.asyncio
    async def test_updates_stored_up_to_capacity(self):
        with gr.Blocks() as demo:
            min = gr.Number()
            num = gr.Number()

            def run(min, num):
                return min, gr.Number(value=num, minimum=min)

            num.submit(
                run,
                inputs=[min, num],
                outputs=[min, num],
                api_name="predict",
            )
        app, _, _ = demo.launch(prevent_thread_lock=True, state_session_capacity=2)
        client = TestClient(app)

        session_1 = client.post(
            f"{API_PREFIX}/api/predict/",
            json={"data": [5, 5], "session_hash": "1", "fn_index": 0},
        )
        assert session_1.json()["data"][0] == 5
        session_1 = client.post(
            f"{API_PREFIX}/api/predict/",
            json={"data": [2, 2], "session_hash": "1", "fn_index": 0},
        )
        assert "error" in session_1.json()  # error because min is 5 and num is 2
        session_2 = client.post(
            f"{API_PREFIX}/api/predict/",
            json={"data": [5, 5], "session_hash": "2", "fn_index": 0},
        )
        assert session_2.json()["data"][0] == 5
        session_3 = client.post(
            f"{API_PREFIX}/api/predict/",
            json={"data": [5, 5], "session_hash": "3", "fn_index": 0},
        )
        assert session_3.json()["data"][0] == 5
        session_1 = client.post(
            f"{API_PREFIX}/api/predict/",
            json={"data": [2, 2], "session_hash": "1", "fn_index": 0},
        )
        assert (
            "error" not in session_1.json()
        )  # no error because sesssion 1 block config was lost when session 3 was added

    def test_state_holder_is_used_in_postprocess(self, connect):
        with gr.Blocks() as demo:
            dropdown = gr.Dropdown(label="list", choices=["Choice 1"], interactive=True)
            button = gr.Button("Get dropdown value")
            button2 = gr.Button("Convert dropdown to multiselect")
            button.click(
                lambda x: x, inputs=dropdown, outputs=dropdown, api_name="predict"
            )
            button2.click(
                lambda: gr.Dropdown(multiselect=True),
                outputs=dropdown,
                api_name="set_multiselect",
            )

        client: Client
        with connect(demo) as client:
            assert client.predict("Choice 1", api_name="/predict") == "Choice 1"
            client.predict(api_name="/set_multiselect")
            assert client.predict("Choice 1", api_name="/predict") == ["Choice 1"]


class TestCallFunction:
    @pytest.mark.asyncio
    async def test_call_regular_function(self):
        with gr.Blocks() as demo:
            text = gr.Textbox()
            btn = gr.Button()
            btn.click(
                lambda x: f"Hello, {x}",
                inputs=text,
                outputs=text,
            )

        output = await demo.call_function(0, ["World"])
        assert output["prediction"] == "Hello, World"
        output = demo("World")
        assert output == "Hello, World"

        output = await demo.call_function(0, ["Abubakar"])
        assert output["prediction"] == "Hello, Abubakar"

    @pytest.mark.asyncio
    async def test_call_multiple_functions(self):
        with gr.Blocks() as demo:
            text = gr.Textbox()
            text2 = gr.Textbox()
            btn = gr.Button()
            btn.click(
                lambda x: f"Hello, {x}",
                inputs=text,
                outputs=text,
            )
            text.change(
                lambda x: f"Hi, {x}",
                inputs=text,
                outputs=text2,
            )

        output = await demo.call_function(0, ["World"])
        assert output["prediction"] == "Hello, World"
        output = demo("World")
        assert output == "Hello, World"

        output = await demo.call_function(1, ["World"])
        assert output["prediction"] == "Hi, World"
        output = demo("World", fn_index=1)  # fn_index must be a keyword argument
        assert output == "Hi, World"

    @pytest.mark.asyncio
    async def test_call_decorated_functions(self):
        with gr.Blocks() as demo:
            name = gr.Textbox(value="Abubakar")
            output = gr.Textbox(label="Output Box")

            @name.submit(inputs=name, outputs=output)
            @demo.load(inputs=name, outputs=output)
            def test(x):
                return "Hello " + x

        output = await demo.call_function(0, ["Adam"])
        assert output["prediction"] == "Hello Adam"
        output = await demo.call_function(1, ["Adam"])
        assert output["prediction"] == "Hello Adam"

    @pytest.mark.asyncio
    async def test_call_generator(self):
        def generator(x):
            yield from range(x)

        with gr.Blocks() as demo:
            inp = gr.Number()
            out = gr.Number()
            btn = gr.Button()
            btn.click(
                generator,
                inputs=inp,
                outputs=out,
            )

        demo.queue()
        assert demo.config["enable_queue"]

        output = await demo.call_function(0, [3])
        assert output["prediction"] == 0
        output = await demo.call_function(0, [3], iterator=output["iterator"])
        assert output["prediction"] == 1
        output = await demo.call_function(0, [3], iterator=output["iterator"])
        assert output["prediction"] == 2
        output = await demo.call_function(0, [3], iterator=output["iterator"])
        assert output["prediction"] == gr.components._Keywords.FINISHED_ITERATING
        assert output["iterator"] is None
        output = await demo.call_function(0, [3], iterator=output["iterator"])
        assert output["prediction"] == 0

    @pytest.mark.asyncio
    async def test_call_both_generator_and_function(self):
        def generator(x):
            for i in range(x):
                yield i, x

        with gr.Blocks() as demo:
            inp = gr.Number()
            out1 = gr.Number()
            out2 = gr.Number()
            btn = gr.Button()
            inp.change(lambda x: x + x, inp, out1)
            btn.click(
                generator,
                inputs=inp,
                outputs=[out1, out2],
            )

        demo.queue()

        output = await demo.call_function(0, [2])
        assert output["prediction"] == 4
        output = await demo.call_function(0, [-1])
        assert output["prediction"] == -2

        output = await demo.call_function(1, [3])
        assert output["prediction"] == (0, 3)
        output = await demo.call_function(1, [3], iterator=output["iterator"])
        assert output["prediction"] == (1, 3)
        output = await demo.call_function(1, [3], iterator=output["iterator"])
        assert output["prediction"] == (2, 3)
        output = await demo.call_function(1, [3], iterator=output["iterator"])
        assert output["prediction"] == (gr.components._Keywords.FINISHED_ITERATING,) * 2
        assert output["iterator"] is None
        output = await demo.call_function(1, [3], iterator=output["iterator"])
        assert output["prediction"] == (0, 3)


class TestBatchProcessing:
    @pytest.mark.asyncio
    async def test_call_regular_function(self):
        def batch_fn(x):
            results = []
            for word in x:
                results.append(f"Hello {word}")
            return (results,)

        with gr.Blocks() as demo:
            text = gr.Textbox()
            btn = gr.Button()
            btn.click(batch_fn, inputs=text, outputs=text, batch=True)

        output = await demo.call_function(0, [["Adam", "Yahya"]])
        assert output["prediction"][0] == ["Hello Adam", "Hello Yahya"]
        output = demo("Abubakar")
        assert output == "Hello Abubakar"

    @pytest.mark.asyncio
    async def test_functions_multiple_parameters(self):
        def regular_fn(word1, word2):
            return len(word1) > len(word2)

        def batch_fn(words, lengths):
            comparisons = []
            trim_words = []
            for word, length in zip(words, lengths, strict=False):
                trim_words.append(word[:length])
                comparisons.append(len(word) > length)
            return trim_words, comparisons

        with gr.Blocks() as demo:
            text1 = gr.Textbox()
            text2 = gr.Textbox()
            leng = gr.Number(precision=0)
            bigger = gr.Checkbox()
            btn1 = gr.Button("Check")
            btn2 = gr.Button("Trim")
            btn1.click(regular_fn, inputs=[text1, text2], outputs=bigger)
            btn2.click(
                batch_fn,
                inputs=[text1, leng],
                outputs=[text1, bigger],
                batch=True,
            )

        output = await demo.call_function(0, ["Adam", "Yahya"])
        assert output["prediction"] is False
        output = demo("Abubakar", "Abid")
        assert output

        output = await demo.call_function(1, [["Adam", "Mary"], [3, 5]])
        assert output["prediction"] == (
            ["Ada", "Mary"],
            [True, False],
        )
        output = demo("Abubakar", 3, fn_index=1)
        assert output == ["Abu", True]

    @pytest.mark.asyncio
    async def test_invalid_batch_generator(self):
        with pytest.raises(ValueError):

            def batch_fn(x):
                results = []
                for word in x:
                    results.append(f"Hello {word}")
                    yield (results,)

            with gr.Blocks() as demo:
                text = gr.Textbox()
                btn = gr.Button()
                btn.click(batch_fn, inputs=text, outputs=text, batch=True)

            await demo.process_api(0, [["Adam", "Yahya"]], state=None)

    @pytest.mark.asyncio
    async def test_exceeds_max_batch_size(self):
        with pytest.raises(ValueError):

            def batch_fn(x):
                results = []
                for word in x:
                    results.append(f"Hello {word}")
                return (results,)

            with gr.Blocks() as demo:
                text = gr.Textbox()
                btn = gr.Button()
                btn.click(
                    batch_fn, inputs=text, outputs=text, batch=True, max_batch_size=2
                )

            await demo.process_api(0, [["A", "B", "C"]], state=None)

    @pytest.mark.asyncio
    async def test_unequal_batch_sizes(self):
        with pytest.raises(ValueError):

            def batch_fn(x, y):
                results = []
                for word1, word2 in zip(x, y, strict=False):
                    results.append(f"Hello {word1}{word2}")
                return (results,)

            with gr.Blocks() as demo:
                t1 = gr.Textbox()
                t2 = gr.Textbox()
                btn = gr.Button()
                btn.click(batch_fn, inputs=[t1, t2], outputs=t1, batch=True)

            await demo.process_api(0, [["A", "B", "C"], ["D", "E"]], state=None)


class TestUpdate:
    @pytest.mark.asyncio
    async def test_accordion_update(self):
        with gr.Blocks() as demo:
            with gr.Accordion(label="Open for greeting", open=False) as accordion:
                gr.Textbox("Hello!")
            open_btn = gr.Button("Open Accordion")
            close_btn = gr.Button("Close Accordion")
            open_btn.click(
                lambda: gr.Accordion(open=True, label="Open Accordion"),
                inputs=None,
                outputs=[accordion],
            )
            close_btn.click(
                lambda: gr.Accordion(open=False, label="Closed Accordion"),
                inputs=None,
                outputs=[accordion],
            )
        result = await demo.process_api(
            block_fn=0, inputs=[None], request=None, state=None
        )
        assert result["data"][0] == {
            "open": True,
            "label": "Open Accordion",
            "__type__": "update",
        }
        result = await demo.process_api(
            block_fn=1, inputs=[None], request=None, state=None
        )
        assert result["data"][0] == {
            "open": False,
            "label": "Closed Accordion",
            "__type__": "update",
        }


@pytest.mark.asyncio
async def test_root_path():
    image_file = pathlib.Path(__file__).parent / "test_files" / "bus.png"
    demo = gr.Interface(lambda x: image_file, "textbox", "image")
    result = await demo.process_api(block_fn=0, inputs=[""], request=None, state=None)
    result_url = result["data"][0]["url"]
    assert result_url.startswith(f"{API_PREFIX}/file=")
    assert result_url.endswith("bus.png")

    result = await demo.process_api(
        block_fn=0, inputs=[""], request=None, state=None, root_path="abidlabs.hf.space"
    )
    result_url = result["data"][0]["url"]
    assert result_url.startswith(f"abidlabs.hf.space{API_PREFIX}/file=")
    assert result_url.endswith("bus.png")


class TestRender:
    def test_duplicate_error(self):
        with pytest.raises(DuplicateBlockError):
            t = gr.Textbox()
            with gr.Blocks():
                t.render()
                gr.Number()
                t.render()

        with pytest.raises(DuplicateBlockError):
            with gr.Blocks():
                t = gr.Textbox()
                t.render()

        with pytest.raises(DuplicateBlockError):
            io = gr.Interface(lambda x: x, gr.Textbox(), gr.Textbox())
            with gr.Blocks():
                io.render()
                io.render()

        with pytest.raises(DuplicateBlockError):
            t = gr.Textbox()
            io = gr.Interface(lambda x: x, t, gr.Textbox())
            with gr.Blocks():
                io.render()
                t.render()

    def test_no_error(self):
        t = gr.Textbox()
        t2 = gr.Textbox()
        with gr.Blocks():
            t.render()
            t3 = t2.render()
        assert t2 == t3

        t = gr.Textbox()
        io = gr.Interface(lambda x: x, t, gr.Textbox())
        with gr.Blocks():
            io.render()
            gr.Textbox()

        io = gr.Interface(lambda x: x, gr.Textbox(), gr.Textbox())
        io2 = gr.Interface(lambda x: x, gr.Textbox(), gr.Textbox())
        with gr.Blocks():
            io.render()
            io3 = io2.render()
        assert io2 == io3

    def test_is_rendered(self):
        t = gr.Textbox()
        with gr.Blocks():
            pass
        assert not t.is_rendered

        t = gr.Textbox()
        with gr.Blocks():
            t.render()
        assert t.is_rendered

        t = gr.Textbox()
        with gr.Blocks():
            t.render()
            t.unrender()
        assert not t.is_rendered

        with gr.Blocks():
            t = gr.Textbox()
        assert t.is_rendered

        with gr.Blocks():
            t = gr.Textbox()
        with gr.Blocks():
            pass
        assert t.is_rendered

        t = gr.Textbox()
        gr.Interface(lambda x: x, "textbox", t)
        assert t.is_rendered

    def test_no_error_if_state_rendered_multiple_times(self):
        state = gr.State("")
        gr.TabbedInterface(
            [
                gr.Interface(
                    lambda _, x: (x, "I don't know"),
                    inputs=[state, gr.Textbox()],
                    outputs=[state, gr.Textbox()],
                ),
                gr.Interface(
                    lambda s: (s, f"User question: {s}"),
                    inputs=[state],
                    outputs=[state, gr.Textbox(interactive=False)],
                ),
            ],
            ["Ask question", "Show question"],
        )

    def test_unrender_in_different_blocks_context(self):
        def count_key_value(obj, key, value):
            """
            Recursively count how many times `obj[key] == value` appears in a nested structure.
            """
            count = 0
            if isinstance(obj, dict):
                if obj.get(key) == value:
                    count += 1
                for v in obj.values():
                    count += count_key_value(v, key, value)
            elif isinstance(obj, list):
                for item in obj:
                    count += count_key_value(item, key, value)
            return count

        with gr.Blocks() as demo:
            with gr.Row() as row1:
                textbox = gr.Textbox()
                assert textbox.parent == row1
            with gr.Row() as row2:  # noqa: F841
                textbox.unrender()
                assert textbox.parent is None
            with gr.Row() as row3:
                textbox.render()
                assert textbox.parent == row3

        # The textbox should be rendered only once
        config = demo.get_config_file()
        assert config and "layout" in config
        assert count_key_value(config["layout"], "id", textbox._id) == 1


class TestCancel:
    @pytest.mark.asyncio
    async def test_cancel_function(self, capsys):
        async def long_job():
            await asyncio.sleep(10)
            print("HELLO FROM LONG JOB")

        with gr.Blocks():
            button = gr.Button(value="Start")
            click = button.click(long_job, None, None)
            cancel = gr.Button(value="Cancel")
            cancel.click(None, None, None, cancels=[click])

        task = asyncio.create_task(long_job())
        task.set_name("foo_0<gradio-sep>event")
        # If cancel_fun didn't cancel long_job the message would be printed to the console
        # The test would also take 10 seconds
        await asyncio.gather(task, cancel_tasks({"foo_0"}), return_exceptions=True)
        captured = capsys.readouterr()
        assert "HELLO FROM LONG JOB" not in captured.out

    @pytest.mark.asyncio
    async def test_cancel_function_with_multiple_blocks(self, capsys):
        async def long_job():
            await asyncio.sleep(10)
            print("HELLO FROM LONG JOB")

        with gr.Blocks() as demo1:
            textbox = gr.Textbox()
            button1 = gr.Button(value="Start")
            button1.click(lambda x: x, textbox, textbox)
        with gr.Blocks() as demo2:
            button2 = gr.Button(value="Start")
            click = button2.click(long_job, None, None)
            cancel = gr.Button(value="Cancel")
            cancel.click(None, None, None, cancels=[click])

        with gr.Blocks():
            with gr.Tab("Demo 1"):
                demo1.render()
            with gr.Tab("Demo 2"):
                demo2.render()

        task = asyncio.create_task(long_job())
        task.set_name("foo_1<gradio-sep>event")
        await asyncio.gather(task, cancel_tasks({"foo_1"}), return_exceptions=True)
        captured = capsys.readouterr()
        assert "HELLO FROM LONG JOB" not in captured.out

    def test_raise_exception_if_cancelling_an_event_thats_not_queued(self):
        def iteration(a):
            yield a

        msg = "Queue needs to be enabled!"

        with pytest.raises(ValueError, match=msg):
            with gr.Blocks() as demo:
                button = gr.Button(value="Predict")
                click = button.click(None, None, None)
                cancel = gr.Button(value="Cancel")
                cancel.click(None, None, None, cancels=[click], queue=False)
            demo.launch(prevent_thread_lock=True)

        with pytest.raises(ValueError, match=msg):
            with gr.Blocks() as demo:
                button = gr.Button(value="Predict")
                click = button.click(None, None, None, queue=False)
                cancel = gr.Button(value="Cancel")
                cancel.click(None, None, None, cancels=[click])
            demo.queue().launch(prevent_thread_lock=True)

    def test_cancel_closes_generator(self):
        """The /cancel endpoint must call close() on server-side generators."""
        from gradio.routes import App
        from gradio.utils import SyncToAsyncIterator

        closed = []

        def gen():
            try:
                while True:
                    yield "running"
            finally:
                closed.append(True)

        with gr.Blocks() as demo:
            out = gr.Textbox()
            btn = gr.Button()
            btn.click(gen, None, out, api_name="predict")

        app = App.create_app(demo)

        event_id = "test_event"
        g = gen()
        next(g)  # advance so GeneratorExit/finally will fire on close
        iterator = SyncToAsyncIterator(g, limiter=None)
        app.iterators[event_id] = iterator

        client = TestClient(app)
        resp = client.post(
            f"{API_PREFIX}/cancel",
            json={
                "session_hash": "test",
                "fn_index": 0,
                "event_id": event_id,
            },
        )
        assert resp.status_code == 200
        assert resp.json()["success"]
        assert closed, "Generator was not closed by /cancel endpoint"
        assert event_id not in app.iterators
        assert event_id in app.iterators_to_reset


class TestGetAPIInfo:
    def test_many_endpoints(self):
        with gr.Blocks() as demo:
            t1 = gr.Textbox()
            t2 = gr.Textbox()
            t3 = gr.Textbox()
            t4 = gr.Textbox()
            t5 = gr.Textbox()
            t1.change(lambda x: x, t1, t2, api_name="change1")
            t2.change(lambda x: x, t2, t3, api_name="change2")
            t3.change(lambda x: x, t3, t4, api_visibility="private")
            t4.change(lambda x: x, t4, t5, api_visibility="private")

        api_info = demo.get_api_info()
        assert api_info
        assert len(api_info["named_endpoints"]) == 2
        assert len(api_info["unnamed_endpoints"]) == 0

    def test_no_endpoints(self):
        with gr.Blocks() as demo:
            t1 = gr.Textbox()
            t2 = gr.Textbox()
            t1.change(lambda x: x, t1, t2, api_visibility="private")

        api_info = demo.get_api_info()
        assert api_info
        assert len(api_info["named_endpoints"]) == 0
        assert len(api_info["unnamed_endpoints"]) == 0


class TestAddRequests:
    def test_no_type_hints(self):
        def moo(a, b):
            return a + b

        inputs = [1, 2]
        request = gr.Request()
        inputs_ = helpers.special_args(moo, copy.deepcopy(inputs), request)[0]
        assert inputs_ == inputs

        boo = partial(moo, a=1)
        inputs = [2]
        inputs_ = helpers.special_args(boo, copy.deepcopy(inputs), request)[0]
        assert inputs_ == inputs

    def test_no_type_hints_with_request(self):
        def moo(a: str, b: int):
            return a + str(b)

        inputs = ["abc", 2]
        request = gr.Request()
        inputs_ = helpers.special_args(moo, copy.deepcopy(inputs), request)[0]
        assert inputs_ == inputs

        boo = partial(moo, a="def")
        inputs = [2]
        inputs_ = helpers.special_args(boo, copy.deepcopy(inputs), request)[0]
        assert inputs_ == inputs

    def test_type_hints_with_request(self):
        def moo2(a: str, b: gr.Request):
            return a

        inputs = ["abc"]
        request = gr.Request()
        inputs_ = helpers.special_args(moo2, copy.deepcopy(inputs), request)[0]
        assert inputs_ == inputs + [request]

        def moo(a: gr.Request, b, c: int):
            return c

        inputs = ["abc", 5]
        request = gr.Request()
        inputs_ = helpers.special_args(moo, copy.deepcopy(inputs), request)[0]
        assert inputs_ == [request] + inputs

    def test_type_hints_with_multiple_requests(self):
        def moo2(a: str, b: gr.Request, c: gr.Request):
            return a

        inputs = ["abc"]
        request = gr.Request()
        inputs_ = helpers.special_args(moo2, copy.deepcopy(inputs), request)[0]
        assert inputs_ == inputs + [request, request]

        def moo(a: gr.Request, b, c: int, d: gr.Request):
            return c

        inputs = ["abc", 5]
        request = gr.Request()
        inputs_ = helpers.special_args(moo, copy.deepcopy(inputs), request)[0]
        assert inputs_ == [request] + inputs + [request]

    def test_default_args(self):
        def moo(a, b, c=42):
            return a + b + c

        inputs = [1, 2]
        request = gr.Request()
        inputs_ = helpers.special_args(moo, copy.deepcopy(inputs), request)[0]
        assert inputs_ == inputs + [42]

        inputs = [1, 2, 24]
        request = gr.Request()
        inputs_ = helpers.special_args(moo, copy.deepcopy(inputs), request)[0]
        assert inputs_ == inputs

    def test_default_args_with_progress(self):
        pr = gr.Progress()

        def moo2(a, b, c=42, pr=pr):
            return a + b + c

        inputs = [1, 2]
        request = gr.Request()
        inputs_, progress_index, *_ = helpers.special_args(
            moo2, copy.deepcopy(inputs), request
        )
        assert inputs_ == inputs + [42, pr]
        assert progress_index == 3

        inputs = [1, 2, 24]
        request = gr.Request()
        inputs_, progress_index, *_ = helpers.special_args(
            moo2, copy.deepcopy(inputs), request
        )
        assert inputs_ == inputs + [pr]
        assert progress_index == 3

        def moo(a, b, pr=pr, c=42):
            return a + b + c

        inputs = [1, 2]
        request = gr.Request()
        inputs_, progress_index, *_ = helpers.special_args(
            moo, copy.deepcopy(inputs), request
        )
        assert inputs_ == inputs + [pr, 42]
        assert progress_index == 2

    def test_default_args_with_request(self):
        pr = gr.Progress()

        def moo2(a, b, req: gr.Request, c=42):
            return a + b + c

        inputs = [1, 2]
        request = gr.Request()
        inputs_ = helpers.special_args(moo2, copy.deepcopy(inputs), request)[0]
        assert inputs_ == inputs + [request, 42]

        def moo(a, b, req: gr.Request, c=42, pr=pr):
            return a + b + c

        inputs = [1, 2]
        request = gr.Request()
        inputs_, progress_index, *_ = helpers.special_args(
            moo, copy.deepcopy(inputs), request
        )
        assert inputs_ == inputs + [request, 42, pr]
        assert progress_index == 4

    def test_default_args_with_event_data(self):
        pr = gr.Progress()
        target = gr.Textbox()

        def moo(a, b, ed: SelectData, c=42):
            return a + b + c

        event_data = SelectData(target=target, data={"index": 24, "value": "foo"})
        inputs = [1, 2]
        request = gr.Request()
        inputs_ = helpers.special_args(moo, copy.deepcopy(inputs), request, event_data)[
            0
        ]
        assert len(inputs_) == 4
        new_event_data = inputs_[2]
        assert inputs_ == inputs + [new_event_data, 42]
        assert isinstance(new_event_data, SelectData)
        assert new_event_data.target == target
        assert new_event_data.index == 24
        assert new_event_data.value == "foo"

        def moo2(a, b, ed: SelectData, c=42, pr=pr):
            return a + b + c

        inputs = [1, 2]
        request = gr.Request()
        inputs_, progress_index, *_ = helpers.special_args(
            moo2, copy.deepcopy(inputs), request, event_data
        )
        assert len(inputs_) == 5
        new_event_data = inputs_[2]
        assert inputs_ == inputs + [new_event_data, 42, pr]
        assert progress_index == 4
        assert isinstance(new_event_data, SelectData)
        assert new_event_data.target == target
        assert new_event_data.index == 24
        assert new_event_data.value == "foo"


@pytest.mark.asyncio
async def test_queue_when_using_auth():
    sleep_time = 1

    async def say_hello(name):
        await asyncio.sleep(sleep_time)
        return f"Hello {name}!"

    with gr.Blocks() as demo:
        _input = gr.Textbox()
        _output = gr.Textbox()
        button = gr.Button()
        button.click(say_hello, _input, _output)
    demo.queue()
    app, _, _ = demo.launch(auth=("abc", "123"), prevent_thread_lock=True)

    with pytest.raises(ValueError):
        grc.Client(f"http://localhost:{demo.server_port}")

    client = grc.Client(f"http://localhost:{demo.server_port}", auth=("abc", "123"))
    jobs = []
    for i in range(3):
        jobs.append(client.submit(f"World {i}", fn_index=0))

    for i, job in enumerate(jobs):
        assert job.result() == f"Hello World {i}!"


def test_temp_file_sets_get_extended():
    test_file_dir = pathlib.Path(pathlib.Path(__file__).parent, "test_files")

    with gr.Blocks() as demo1:
        gr.Video(str(test_file_dir / "video_sample.mp4"))

    with gr.Blocks() as demo2:
        gr.Audio(str(test_file_dir / "audio_sample.wav"))

    with gr.Blocks() as demo3:
        demo1.render()
        demo2.render()
    # The upload_set is empty so we remove it from the check
    demo_3_no_empty = [s for s in demo3.temp_file_sets if len(s)]
    demo_1_and_2_no_empty = [
        s for s in demo1.temp_file_sets + demo2.temp_file_sets if len(s)
    ]
    assert demo_3_no_empty == demo_1_and_2_no_empty


def test_recover_kwargs():
    audio = gr.Audio(format="wav", autoplay=True)
    props = audio.recover_kwargs(
        {"format": "wav", "value": "foo.wav", "autoplay": False, "foo": "bar"}
    )
    assert props == {"format": "wav", "value": "foo.wav", "autoplay": False}
    props = audio.recover_kwargs(
        {"format": "wav", "value": "foo.wav", "autoplay": False, "foo": "bar"},
        ["value"],
    )
    assert props == {"format": "wav", "autoplay": False}


def test_postprocess_update_dict():
    block = gr.Textbox()
    update_dict = {"value": 2.0, "visible": True, "invalid_arg": "hello"}
    assert blocks.postprocess_update_dict(block, update_dict, True) == {
        "__type__": "update",
        "value": "2.0",
        "visible": True,
    }

    block = gr.Textbox(lines=10)
    update_dict = {"value": 2.0, "lines": 10}
    assert blocks.postprocess_update_dict(block, update_dict, False) == {
        "__type__": "update",
        "value": 2.0,
        "lines": 10,
    }

    block = gr.Dropdown(choices=["New Country A", "New Country B"])
    update_dict = {
        "value": "New Country A",
        "choices": ["New Country A", "New Country B"],
    }
    assert blocks.postprocess_update_dict(block, update_dict, False) == {
        "__type__": "update",
        "value": "New Country A",
        "choices": [
            ("New Country A", "New Country A"),
            ("New Country B", "New Country B"),
        ],
    }


def test_async_iterator_update_with_new_component(connect):
    async def get_number_stream():
        for i in range(10):
            yield gr.Number(value=i, label="Number (updates every second)")

            await asyncio.sleep(0.1)

    demo = gr.Interface(
        fn=get_number_stream, inputs=None, outputs=["number"], api_name="predict"
    )
    demo.queue()

    with connect(demo) as client:
        job = client.submit(api_name="/predict")
        job.result()
        assert [r["value"] for r in job.outputs()] == list(range(10))


def test_emptry_string_api_name_gets_set_as_fn_name():
    def test_fn(x):
        return x

    with gr.Blocks() as demo:
        t1 = gr.Textbox()
        t2 = gr.Textbox()
        t1.change(test_fn, t1, t2, api_name="")

    assert demo.fns[0].api_name == "test_fn"


@pytest.mark.asyncio
async def test_blocks_postprocessing_with_copies_of_component_instance():
    # Test for: https://github.com/gradio-app/gradio/issues/6608
    with gr.Blocks() as demo:
        chatbot = gr.Chatbot()
        chatbot2 = gr.Chatbot()
        chatbot3 = gr.Chatbot()
        clear = gr.Button("Clear")

        def clear_func():
            return tuple([gr.Chatbot(value=[])] * 3)

        clear.click(
            fn=clear_func, outputs=[chatbot, chatbot2, chatbot3], api_name="clear"
        )

        output = await demo.postprocess_data(
            demo.fns[0], [gr.Chatbot(value=[])] * 3, None
        )
        assert output == [{"value": [], "__type__": "update"}] * 3


def test_static_files_single_app(connect, gradio_temp_dir):
    gr.set_static_paths(
        paths=["test/test_files/cheetah1.jpg", "test/test_files/bus.png"]
    )
    demo = gr.Interface(
        lambda s: s.rotate(45),
        gr.Image(value="test/test_files/cheetah1.jpg", type="pil"),
        gr.Image(),
        examples=["test/test_files/bus.png"],
    )

    # Nothing got saved to cache
    assert len(list(gradio_temp_dir.glob("**/*.*"))) == 0

    with connect(demo) as client:
        client.predict(grc.handle_file("test/test_files/bus.png"))

    # Input/Output got saved to cache
    assert len(list(gradio_temp_dir.glob("**/*.*"))) == 2


def test_static_files_multiple_apps(gradio_temp_dir):
    gr.set_static_paths(paths=["test/test_files/cheetah1.jpg"])
    demo = gr.Interface(
        lambda s: s.rotate(45),
        gr.Image(value="test/test_files/cheetah1.jpg"),
        gr.Image(),
    )

    gr.set_static_paths(paths=["test/test_files/images"])
    demo_2 = gr.Interface(
        lambda s: s.rotate(45),
        gr.Image(value="test/test_files/images/bus.png"),
        gr.Image(),
    )

    with gr.Blocks():
        demo.render()
        demo_2.render()

    # Input/Output got saved to cache
    assert len(list(gradio_temp_dir.glob("**/*.*"))) == 0


def test_time_to_live_and_delete_callback_for_state(capsys, monkeypatch):
    monkeypatch.setenv("GRADIO_IS_E2E_TEST", "1")

    def test_fn(x):
        return x + 1, x + 1

    def delete_fn(v):
        print(f"deleted {v}")

    with gr.Blocks() as demo:
        n1 = gr.Number(value=0)
        state = gr.State(
            value=0, time_to_live=1, delete_callback=lambda v: delete_fn(v)
        )
        button = gr.Button("Increment")
        button.click(test_fn, [state], [n1, state], api_name="increment")

    app, url, _ = demo.launch(prevent_thread_lock=True)

    try:
        client_1 = grc.Client(url)
        client_2 = grc.Client(url)

        client_1.predict(api_name="/increment")
        client_1.predict(api_name="/increment")
        client_1.predict(api_name="/increment")

        client_2.predict(api_name="/increment")
        client_2.predict(api_name="/increment")

        time.sleep(3)

        captured = capsys.readouterr()
        assert "deleted 2" in captured.out
        assert "deleted 3" in captured.out
        for client in [client_1, client_2]:
            assert (
                len(app.state_holder.session_data[client.session_hash].state_data) == 0  # type: ignore
            )
    finally:
        demo.close()


def test_post_process_file_blocked(connect):
    dotfile = pathlib.Path(".foo.txt")
    file = pathlib.Path(os.getcwd()) / ".." / "file.txt"

    try:
        demo = gr.Interface(lambda s: s, "text", "file")
        with connect(demo, show_error=True) as client:
            _ = client.predict("test/test_files/bus.png")
            with pytest.raises(
                ValueError,
                match="to the gradio cache dir because it was not created by",
            ):
                file.write_text("Hi")
                client.predict(str(file))

        with connect(demo, allowed_paths=[str(file)]) as client:
            _ = client.predict(str(file))

        dotfile.write_text("foo")
        with connect(demo, show_error=True) as client:
            with pytest.raises(ValueError, match="Dotfiles located"):
                _ = client.predict(str(dotfile))

        with connect(demo, allowed_paths=[str(dotfile)]) as client:
            _ = client.predict(str(dotfile))

    finally:
        try:
            dotfile.unlink()
        except FileNotFoundError:
            pass


def test_render_when_mounted_sets_root_path_for_files():
    app = FastAPI()
    test_video_path = "test/test_files/video_sample.mp4"

    with gr.Blocks() as demo:
        text = gr.Text()
        gr.Video(test_video_path)

        @gr.render(inputs=text)
        def show_video(data):
            gr.Video(test_video_path)

    app = gr.mount_gradio_app(app, demo, path="/test")

    with TestClient(app) as client:
        r = client.post(
            f"/test{API_PREFIX}/queue/join",
            json={
                "data": [""],
                "fn_index": 0,
                "event_data": None,
                "session_hash": "foo",
                "trigger_id": None,
            },
        )
        assert r.status_code == 200
        r = client.get(f"/test{API_PREFIX}/queue/data?session_hash=foo")
        checked_component = False
        for msg in r.iter_lines():
            if "data" in msg:
                data = json.loads(msg[5:])
                if data["msg"] == "process_completed":
                    render_config = data["output"]["render_config"]
                    for component in render_config["components"]:
                        if "value" in component.get("props", {}):
                            assert component["props"]["value"]["url"].startswith(
                                f"http://testserver/test{API_PREFIX}/file="
                            )
                            checked_component = True
        assert checked_component


@pytest.fixture
def mock_css_files():
    css_contents = {
        "file1.css": "h1 { font-size: 20px; }",
        "file2.css": ".class { margin: 10px; }",
    }

    def mock_open_file(filename, encoding):
        return mock_open(read_data=css_contents[filename])()

    with patch("builtins.open", side_effect=mock_open_file):
        yield


def test_css_and_css_paths_parameters(gradio_temp_dir):
    (gradio_temp_dir / "file1.css").write_text("h1 { font-size: 20px; }")
    (gradio_temp_dir / "file2.css").write_text(".class { margin: 10px; }")
    css_paths = [str(gradio_temp_dir / "file1.css"), str(gradio_temp_dir / "file2.css")]
    expected_css = """
body { color: red; }
h1 { font-size: 20px; }
.class { margin: 10px; }
        """
    with gr.Blocks() as instance:
        pass
    instance.launch(
        prevent_thread_lock=True, css="body { color: red; }", css_paths=css_paths
    )
    assert instance.css is not None
    assert instance.css.strip() == expected_css.strip()
    instance.close()


def test_navbar_config():
    """
    Test that navbar component produces the correct config
    """
    with gr.Blocks() as demo:
        gr.Navbar([("About2", "/about2")], visible=True, main_page_name="My Custom App")
        gr.Textbox(label="Main page content")

    with demo.route("About"):
        gr.Markdown("About page")

    config = demo.get_config_file()
    navbar_component = None
    for component in config["components"]:
        if component["type"] == "navbar":
            navbar_component = component
            break

    assert navbar_component is not None
    assert navbar_component["props"]["value"] == [["About2", "/about2"]]
    assert navbar_component["props"]["visible"]
    assert navbar_component["props"]["main_page_name"] == "My Custom App"


def test_multiple_navbar_components_in_same_page_raise_error():
    # This SHOULD raise an error since we have multiple navbar components in the same page
    with pytest.raises(ValueError):
        with gr.Blocks() as demo:
            gr.Navbar()
            gr.Textbox()
            gr.Navbar()  # This should raise an error
            gr.Textbox()

    # This should NOT raise an error since each navbar is on a different page
    with gr.Blocks() as demo:
        gr.Navbar()
        gr.Textbox()
    with demo.route("Page 2"):
        gr.Navbar()
        gr.Textbox()


def test_blocks_close_closes_thread_properly():
    a = gr.Blocks()

    def poll():
        start = time.time()
        while time.time() - start < 1:
            time.sleep(0.25)
        print("Closing...")
        a.close()

    t = threading.Thread(target=poll, daemon=True)
    t.start()

    with a:
        gr.Markdown("Testing close")
    a.launch(prevent_thread_lock=True)

    time.sleep(1.2)
    assert not t.is_alive()
    assert not a.is_running



import os
import tempfile
import textwrap
from pathlib import Path
from unittest.mock import MagicMock, patch

import huggingface_hub
import pytest

import gradio as gr
from gradio.context import Context
from gradio.exceptions import GradioVersionIncompatibleError, InvalidApiNameError
from gradio.external import TooManyRequestsError
from gradio.external_utils import cols_to_rows, get_tabular_examples

"""
WARNING: These tests have an external dependency: namely that Hugging Face's
Hub and Space APIs do not change, and they keep their most famous models up.
So if, e.g. Spaces is down, then these test will not pass.

These tests actually test gr.load() and gr.Blocks.load() but are
included in a separate file because of the above-mentioned dependency.
"""

# Mark the whole module as flaky and serial
pytestmark = [pytest.mark.flaky, pytest.mark.serial]

os.environ["GRADIO_ANALYTICS_ENABLED"] = "False"

HF_TOKEN = huggingface_hub.get_token()


class TestLoadInterface:
    def test_audio_to_audio(self):
        model_type = "audio-to-audio"
        interface = gr.load(
            name="speechbrain/mtl-mimic-voicebank",
            src="models",
            alias=model_type,
        )
        assert interface.__name__ == model_type
        assert interface.input_components and interface.output_components
        assert isinstance(interface.input_components[0], gr.Audio)
        assert isinstance(interface.output_components[0], gr.Audio)

    def test_question_answering(self):
        model_type = "image-classification"
        interface = gr.load(
            name="lysandre/tiny-vit-random",
            src="models",
            alias=model_type,
        )
        assert interface.__name__ == model_type
        assert interface.input_components and interface.output_components
        assert isinstance(interface.input_components[0], gr.Image)
        assert isinstance(interface.output_components[0], gr.Label)

    def test_text_generation(self):
        model_type = "text_generation"
        interface = gr.load(
            "models/gpt2", alias=model_type, description="This is a test description"
        )
        assert interface.__name__ == model_type
        assert interface.input_components and interface.output_components
        assert isinstance(interface.input_components[0], gr.Textbox)
        assert isinstance(interface.output_components[0], gr.Textbox)
        assert any(
            "This is a test description" in d["props"].get("value", "")
            for d in interface.get_config_file()["components"]
        )

    def test_summarization(self):
        model_type = "summarization"
        interface = gr.load(
            "models/facebook/bart-large-cnn", token=HF_TOKEN, alias=model_type
        )
        assert interface.__name__ == model_type
        assert interface.input_components and interface.output_components
        assert isinstance(interface.input_components[0], gr.Textbox)
        assert isinstance(interface.output_components[0], gr.Textbox)

    def test_translation(self):
        model_type = "translation"
        interface = gr.load(
            "models/facebook/bart-large-cnn", token=HF_TOKEN, alias=model_type
        )
        assert interface.__name__ == model_type
        assert interface.input_components and interface.output_components
        assert isinstance(interface.input_components[0], gr.Textbox)
        assert isinstance(interface.output_components[0], gr.Textbox)

    def test_text_classification(self):
        model_type = "text-classification"
        interface = gr.load(
            "models/distilbert-base-uncased-finetuned-sst-2-english",
            token=HF_TOKEN,
            alias=model_type,
        )
        assert interface.__name__ == model_type
        assert interface.input_components and interface.output_components
        assert isinstance(interface.input_components[0], gr.Textbox)
        assert isinstance(interface.output_components[0], gr.Label)

    def test_fill_mask(self):
        model_type = "fill-mask"
        interface = gr.load(
            "models/bert-base-uncased", token=HF_TOKEN, alias=model_type
        )
        assert interface.__name__ == model_type
        assert interface.input_components and interface.output_components
        assert isinstance(interface.input_components[0], gr.Textbox)
        assert isinstance(interface.output_components[0], gr.Label)

    def test_zero_shot_classification(self):
        model_type = "zero-shot-classification"
        interface = gr.load(
            "models/facebook/bart-large-mnli", token=HF_TOKEN, alias=model_type
        )
        assert interface.__name__ == model_type
        assert interface.input_components and interface.output_components
        assert isinstance(interface.input_components[0], gr.Textbox)
        assert isinstance(interface.input_components[1], gr.Textbox)
        assert isinstance(interface.input_components[2], gr.Checkbox)
        assert isinstance(interface.output_components[0], gr.Label)

    def test_automatic_speech_recognition(self):
        model_type = "automatic-speech-recognition"
        interface = gr.load(
            "models/facebook/wav2vec2-base-960h", token=HF_TOKEN, alias=model_type
        )
        assert interface.__name__ == model_type
        assert interface.input_components and interface.output_components
        assert isinstance(interface.input_components[0], gr.Audio)
        assert isinstance(interface.output_components[0], gr.Textbox)

    def test_image_classification(self):
        model_type = "image-classification"
        interface = gr.load(
            "models/google/vit-base-patch16-224", token=HF_TOKEN, alias=model_type
        )
        assert interface.__name__ == model_type
        assert interface.input_components and interface.output_components
        assert isinstance(interface.input_components[0], gr.Image)
        assert isinstance(interface.output_components[0], gr.Label)

    def test_feature_extraction(self):
        model_type = "feature-extraction"
        interface = gr.load(
            "models/sentence-transformers/distilbert-base-nli-mean-tokens",
            token=HF_TOKEN,
            alias=model_type,
        )
        assert interface.__name__ == model_type
        assert interface.input_components and interface.output_components
        assert isinstance(interface.input_components[0], gr.Textbox)
        assert isinstance(interface.output_components[0], gr.Dataframe)

    def test_sentence_similarity(self):
        model_type = "text-to-speech"
        interface = gr.load(
            "models/julien-c/ljspeech_tts_train_tacotron2_raw_phn_tacotron_g2p_en_no_space_train",
            token=HF_TOKEN,
            alias=model_type,
        )
        assert interface.__name__ == model_type
        assert interface.input_components and interface.output_components
        assert isinstance(interface.input_components[0], gr.Textbox)
        assert isinstance(interface.output_components[0], gr.Audio)

    def test_text_to_speech(self):
        model_type = "text-to-speech"
        interface = gr.load(
            "models/julien-c/ljspeech_tts_train_tacotron2_raw_phn_tacotron_g2p_en_no_space_train",
            token=HF_TOKEN,
            alias=model_type,
        )
        assert interface.__name__ == model_type
        assert interface.input_components and interface.output_components
        assert isinstance(interface.input_components[0], gr.Textbox)
        assert isinstance(interface.output_components[0], gr.Audio)

    def test_raise_incompatbile_version_error(self):
        with pytest.raises(GradioVersionIncompatibleError):
            gr.load("spaces/gradio-tests/titanic-survival")

    def test_multiple_spaces_one_private(self):
        with gr.Blocks():
            gr.load(
                "spaces/gradio-tests/not-actually-private-spacev4-sse",
                token=HF_TOKEN,
            )
            gr.load(
                "spaces/gradio/test-loading-examplesv4-sse",
            )
        assert Context.token == HF_TOKEN

    def test_private_space_v4_sse_v1(self):
        io = gr.load(
            "spaces/gradio-tests/not-actually-private-spacev4-sse-v1",
            token=HF_TOKEN,
        )
        try:
            output = io("abc")
            assert output == "abc"
            assert io._deprecated_theme == "gradio/monochrome"
        except TooManyRequestsError:
            pass


class TestLoadInterfaceWithExamples:
    def test_interface_load_examples(self, tmp_path):
        test_file_dir = Path(Path(__file__).parent, "test_files")
        with patch("gradio.utils.get_cache_folder", return_value=tmp_path):
            gr.load(
                name="models/google/vit-base-patch16-224",
                examples=[Path(test_file_dir, "cheetah1.jpg")],
                cache_examples=False,
            )

    def test_interface_load_cache_examples(self, tmp_path):
        test_file_dir = Path(Path(__file__).parent, "test_files")
        with patch(
            "gradio.utils.get_cache_folder", return_value=Path(tempfile.mkdtemp())
        ):
            try:
                gr.load(
                    name="models/google/vit-base-patch16-224",
                    examples=[Path(test_file_dir, "cheetah1.jpg")],
                    cache_examples=True,
                    token=HF_TOKEN,
                )
            except TooManyRequestsError:
                pass

    def test_proxy_url(self):
        demo = gr.load("spaces/gradio/test-loading-examplesv4-sse")
        assert all(
            c["props"]["proxy_url"]
            == "https://gradio-test-loading-examplesv4-sse.hf.space/"
            for c in demo.get_config_file()["components"]
        )

    def test_root_url_deserialization(self):
        demo = gr.load("spaces/gradio/simple_galleryv4-sse")
        gallery = demo("test")
        assert all("caption" in d for d in gallery)

    def test_loading_chatbot_with_avatar_images_does_not_raise_errors(self):
        gr.load("gradio/chatbot_multimodal", src="spaces")


def test_get_tabular_examples_replaces_nan_with_str_nan():
    readme = """
        ---
        tags:
        - sklearn
        - skops
        - tabular-classification
        widget:
          structuredData:
            attribute_0:
            - material_7
            - material_7
            - material_7
            measurement_2:
            - 14.206
            - 15.094
            - .nan
        ---
    """
    mock_response = MagicMock()
    mock_response.status_code = 200
    mock_response.text = textwrap.dedent(readme)

    with patch("gradio.external.httpx.get", return_value=mock_response):
        examples = get_tabular_examples("foo-model")
        assert examples["measurement_2"] == [14.206, 15.094, "NaN"]


def test_cols_to_rows():
    assert cols_to_rows({"a": [1, 2, "NaN"], "b": [1, "NaN", 3]}) == (
        ["a", "b"],
        [[1, 1], [2, "NaN"], ["NaN", 3]],
    )
    assert cols_to_rows({"a": [1, 2, "NaN", 4], "b": [1, "NaN", 3]}) == (
        ["a", "b"],
        [[1, 1], [2, "NaN"], ["NaN", 3], [4, "NaN"]],
    )
    assert cols_to_rows({"a": [1, 2, "NaN"], "b": [1, "NaN", 3, 5]}) == (
        ["a", "b"],
        [[1, 1], [2, "NaN"], ["NaN", 3], ["NaN", 5]],
    )
    assert cols_to_rows({"a": None, "b": [1, "NaN", 3, 5]}) == (
        ["a", "b"],
        [["NaN", 1], ["NaN", "NaN"], ["NaN", 3], ["NaN", 5]],
    )
    assert cols_to_rows({"a": None, "b": None}) == (["a", "b"], [])


def check_dataframe(config):
    input_df = next(
        c for c in config["components"] if c["props"].get("label", "") == "Input Rows"
    )
    assert input_df["props"]["headers"] == ["a", "b"]
    assert input_df["props"]["row_count"] == [3, "dynamic"]
    assert input_df["props"]["col_count"] == [2, "dynamic"]


def check_dataset(config, readme_examples):
    # No Examples
    if not any(readme_examples.values()):
        assert not any(c for c in config["components"] if c["type"] == "dataset")
    else:
        dataset = next(c for c in config["components"] if c["type"] == "dataset")
        assert dataset["props"]["samples"] == [[cols_to_rows(readme_examples)[1]]]


@pytest.mark.parametrize(
    "hypothetical_readme",
    [
        {"a": [1, 2, "NaN"], "b": [1, "NaN", 3]},
        {"a": [1, 2, "NaN", 4], "b": [1, "NaN", 3]},
        {"a": [1, 2, "NaN"], "b": [1, "NaN", 3, 5]},
        {"a": None, "b": [1, "NaN", 3, 5]},
        {"a": None, "b": None},
    ],
)
def test_can_load_tabular_model_with_different_widget_data(hypothetical_readme):
    with patch(
        "gradio.external_utils.get_tabular_examples", return_value=hypothetical_readme
    ):
        io = gr.load("models/scikit-learn/tabular-playground")
        check_dataframe(io.config)
        check_dataset(io.config, hypothetical_readme)


def test_raise_value_error_when_api_name_invalid():
    demo = gr.load(name="spaces/gradio/hello_worldv4-sse")
    with pytest.raises(InvalidApiNameError):
        demo("freddy", api_name="route does not exist")


def test_use_api_name_in_call_method():
    # Interface
    demo = gr.load(name="spaces/gradio/hello_worldv4-sse")
    assert demo("freddy", api_name="predict") == "Hello freddy!"

    # Blocks demo with multiple functions
    # app = gr.load(name="spaces/gradio/multiple-api-name-test")
    # assert app(15, api_name="minus_one") == 14
    # assert app(4, api_name="double") == 8


def test_load_custom_component():
    pytest.skip("Custom components not supported yet")

    from gradio_pdf import PDF  # noqa

    demo = gr.load("spaces/freddyaboulton/gradiopdf")
    output = demo(
        "test/test_files/sample_file.pdf", "What does this say?", api_name="predict"
    )
    assert isinstance(output, str)


def test_load_inside_blocks():
    demo = gr.load("spaces/abidlabs/en2fr")
    output = demo("Hello", api_name="predict")
    assert isinstance(output, str)


def test_load_callable():
    def mock_src(name: str, token: str | None, **kwargs) -> gr.Blocks:
        assert name == "test_model"
        assert token == "test_token"
        assert kwargs == {"param1": "value1", "param2": "value2"}
        return gr.Blocks()

    result = gr.load(
        "test_model",
        mock_src,
        "test_token",
        None,
        param1="value1",
        param2="value2",
    )

    assert isinstance(result, gr.Blocks)


@patch("openai.OpenAI")
def test_load_chat_basic(mock_openai):
    mock_client = MagicMock()
    mock_client.chat.completions.create.return_value.choices[
        0
    ].message.content = "Hello human!"
    mock_openai.return_value = mock_client

    chat = gr.load_chat(
        "http://fake-api.com/v1",
        model="test-model",
        token="fake-token",
        streaming=False,
    )
    response = chat.fn("Hi AI!", None)
    assert response == "Hello human!"


@patch("openai.OpenAI")
def test_load_chat_with_streaming(mock_openai):
    mock_client = MagicMock()
    mock_stream = [
        MagicMock(choices=[MagicMock(delta=MagicMock(content="Hello"))]),
        MagicMock(choices=[MagicMock(delta=MagicMock(content=" World"))]),
        MagicMock(choices=[MagicMock(delta=MagicMock(content="!"))]),
    ]
    mock_client.chat.completions.create.return_value = mock_stream
    mock_openai.return_value = mock_client
    chat = gr.load_chat(
        "http://fake-api.com/v1", model="test-model", token="fake-token", streaming=True
    )
    response_stream = chat.fn("Hi!", None)
    responses = list(response_stream)
    assert responses == ["Hello", "Hello World", "Hello World!"]


def test_load_chat_textbox_override():
    from gradio import ChatInterface

    custom_textbox = gr.Textbox(placeholder="Custom textbox", container=False)
    chat = gr.load_chat(
        base_url="http://localhost:1234/v1/",
        model="demo",
        token="dummy",
        textbox=custom_textbox,
        streaming=False,
    )
    assert isinstance(chat, ChatInterface)
    assert chat.textbox is custom_textbox



from __future__ import annotations

import re
import warnings
from typing import Any


class I18nData:
    """
    A class that wraps a translation key with metadata.

    This object will be serialized and sent to the frontend, where the actual
    translation will happen using the frontend's i18n system.
    """

    def __init__(self, key: str):
        """
        Initialize a I18nData object.

        Args:
            key: The translation key to be translated in the frontend.
        """
        self.key = key
        self._type = "translation_metadata"

    def to_dict(self) -> dict[str, Any]:
        """
        Convert the I18nData object to a dictionary for serialization.
        This allows the frontend to recognize it as a translatable object.
        """
        return {"__type__": self._type, "key": self.key}

    def __str__(self) -> str:
        """
        String representation of the I18nData object.
        Used when the object is converted to a string.
        This returns a special format that can be recognized by the frontend
        as needing translation.
        """
        import json

        return f"__i18n__{json.dumps(self.to_dict())}"

    def __repr__(self) -> str:
        """
        Representation of the I18nData object for debugging.
        """
        return self.__str__()

    def __add__(self, other):
        """
        Handle string concatenation (self + other).
        """
        return str(self) + str(other)

    def __radd__(self, other):
        """
        Handle string concatenation (other + self).
        """
        return str(other) + str(self)

    def __getattr__(self, name):
        """
        Handle attribute access for I18nData.
        This makes it possible to use I18nData objects in contexts
        that expect strings with methods.
        """
        if name.startswith("__") and name.endswith("__"):
            raise AttributeError(f"{self.__class__.__name__} has no attribute {name}")

        def method(*_args, **_kwargs):
            return self

        return method

    def tojson(self) -> dict[str, Any]:
        """
        Convert the I18nData object to a JSON-serializable dictionary.
        This is used by the default Python JSON serializer.
        """
        return self.to_dict()


class I18n:
    """
    Handles internationalization (i18n) for Gradio applications.

    Stores translation dictionaries and provides a method to retrieve translation keys.
    The translation lookup happens on the frontend based on the browser's locale
    and the provided translation dictionaries.
    """

    # BCP 47 language tag regex pattern
    _LOCALE_PATTERN = re.compile(r"^[a-z]{2,3}(-[A-Za-z0-9]{2,8})*$")

    def __init__(self, **translations: dict[str, str]):
        """
        Initializes the I18n class.

        Args:
            **translations: Each keyword argument should be a locale code (e.g., "en", "fr") with a
                          dictionary value, which maps translation keys to translated strings.
                          Example: gr.I18n(en={"greeting": "Hello"}, es={"greeting": "Hola"})

                          These translations can be passed to the frontend for use there.
        """
        self.translations = {}

        for locale, translation_dict in translations.items():
            if not self._is_valid_locale(locale):
                warnings.warn(
                    f"Invalid locale code: '{locale}'. Locale codes should follow BCP 47 format (e.g., 'en', 'en-US'). "
                    f"This locale will still be included, but may not work correctly.",
                    UserWarning,
                )
            self.translations[locale] = translation_dict

    def _is_valid_locale(self, locale: str) -> bool:
        return bool(self._LOCALE_PATTERN.match(locale))

    def __call__(self, key: str) -> I18nData:
        """
        Returns a I18nData object containing the translation key.

        This metadata object will be serialized and sent to the frontend,
        where it will be translated by the frontend's i18n system.

        Args:
            key: The key to identify the translation string (e.g., "submit_button").

        Returns:
            A I18nData object containing the translation key.
        """
        return I18nData(key)

    @property
    def translations_dict(self) -> dict[str, dict[str, str]]:
        """
        Returns the dictionary of translations provided during initialization.
        These can be passed to the frontend for use in its translation system.
        """
        return self.translations



from __future__ import annotations

import os
import socket
import sys
import threading
import time
from collections.abc import Callable
from functools import partial
from typing import TYPE_CHECKING, TypeVar

import uvicorn
from uvicorn.config import Config

from gradio.exceptions import ServerFailedToStartError
from gradio.routes import App
from gradio.utils import (
    ServerReloader,
    SourceFileReloader,
    SpacesReloader,
    get_space,
    watchfn,
    watchfn_spaces,
)

if TYPE_CHECKING:  # Only import for type checking (to avoid circular imports).
    _ServerReloaderT = TypeVar("_ServerReloaderT", bound=ServerReloader)

# By default, the local server will try to open on localhost, port 7860.
# If that is not available, then it will try 7861, 7862, ... 7959.
INITIAL_PORT_VALUE = int(os.getenv("GRADIO_SERVER_PORT", "7860"))
TRY_NUM_PORTS = int(os.getenv("GRADIO_NUM_PORTS", "100"))
LOCALHOST_NAME = os.getenv("GRADIO_SERVER_NAME", "127.0.0.1")

GRADIO_HOT_RELOAD = os.getenv("GRADIO_HOT_RELOAD", "false").lower()

should_watch = bool(os.getenv("GRADIO_WATCH_DIRS", ""))
GRADIO_WATCH_DIRS = (
    os.getenv("GRADIO_WATCH_DIRS", "").split(",") if should_watch else []
)
GRADIO_WATCH_MODULE_NAME = os.getenv("GRADIO_WATCH_MODULE_NAME", "app")
GRADIO_WATCH_DEMO_NAME = os.getenv("GRADIO_WATCH_DEMO_NAME", "")
GRADIO_WATCH_DEMO_PATH = os.getenv("GRADIO_WATCH_DEMO_PATH", "")


class Server(uvicorn.Server):
    def __init__(
        self,
        config: Config,
        reloader: _ServerReloaderT | None = None,
        watchfn: Callable[[_ServerReloaderT], None] = watchfn,  # ty: ignore[invalid-parameter-default]
    ) -> None:
        self.running_app = config.app
        super().__init__(config)
        self.reloader = reloader
        if self.reloader:
            self.event = threading.Event()
            self.watch = partial(watchfn, self.reloader)

    def install_signal_handlers(self):
        pass

    def run_in_thread(self):
        self.thread = threading.Thread(target=self.run, daemon=True)
        if self.reloader:
            self.watch_thread = threading.Thread(target=self.watch, daemon=True)
            self.watch_thread.start()
        self.thread.start()
        start = time.time()
        while not self.started:
            time.sleep(1e-3)
            if time.time() - start > 5:
                raise ServerFailedToStartError(
                    "Server failed to start. Please check that the port is available."
                )

    def close(self):
        self.should_exit = True
        if self.reloader:
            self.reloader.stop()
            self.watch_thread.join()
        self.thread.join(timeout=5)


def start_server(
    app: App,
    server_name: str | None = None,
    server_port: int | None = None,
    ssl_keyfile: str | None = None,
    ssl_certfile: str | None = None,
    ssl_keyfile_password: str | None = None,
) -> tuple[str, int, str, Server]:
    """Launches a local server running the provided Interface
    Parameters:
        app: the FastAPI app object to run
        server_name: to make app accessible on local network, set this to "0.0.0.0". Can be set by environment variable GRADIO_SERVER_NAME.
        server_port: will start gradio app on this port (if available). Can be set by environment variable GRADIO_SERVER_PORT.
        auth: If provided, username and password (or list of username-password tuples) required to access the Blocks. Can also provide function that takes username and password and returns True if valid login.
        ssl_keyfile: If a path to a file is provided, will use this as the private key file to create a local server running on https.
        ssl_certfile: If a path to a file is provided, will use this as the signed certificate for https. Needs to be provided if ssl_keyfile is provided.
        ssl_keyfile_password: If a password is provided, will use this with the ssl certificate for https.

    Returns:
        server_name: the name of the server (default is "localhost")
        port: the port number the server is running on
        path_to_local_server: the complete address that the local server can be accessed at
        server: the server object that is a subclass of uvicorn.Server (used to close the server)
    """
    if ssl_keyfile is not None and ssl_certfile is None:
        raise ValueError("ssl_certfile must be provided if ssl_keyfile is provided.")

    server_name = server_name or LOCALHOST_NAME
    url_host_name = "localhost" if server_name == "0.0.0.0" else server_name

    # Strip IPv6 brackets from the address if they exist.
    # This is needed as http://[::1]:port/ is a valid browser address,
    # but not a valid IPv6 address, so asyncio will throw an exception.
    if server_name.startswith("[") and server_name.endswith("]"):
        host = server_name[1:-1]
    else:
        host = server_name

    server_ports = (
        [server_port]
        if server_port is not None
        else range(INITIAL_PORT_VALUE, INITIAL_PORT_VALUE + TRY_NUM_PORTS)
    )

    for port in server_ports:
        try:
            # The fastest way to check if a port is available is to try to bind to it with socket.
            # If the port is not available, socket will throw an OSError.
            s = socket.socket()
            s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
            # Really, we should be checking if (server_name, server_port) is available, but
            # socket.bind() doesn't seem to throw an OSError with ipv6 addresses, based on my testing.
            # Instead, we just check if the port is available on localhost.
            s.bind((LOCALHOST_NAME, port))
            s.close()

            # To avoid race conditions, so we also check if the port by trying to start the uvicorn server.
            # If the port is not available, this will throw a ServerFailedToStartError.
            config = uvicorn.Config(
                app=app,
                port=port,
                host=host,
                log_level="warning",
                ssl_keyfile=ssl_keyfile,
                ssl_certfile=ssl_certfile,
                ssl_keyfile_password=ssl_keyfile_password,
            )
            if get_space() is not None:
                reloader = SpacesReloader(
                    app=app,
                    watch_dirs=GRADIO_WATCH_DIRS,
                    demo_name=GRADIO_WATCH_DEMO_NAME,
                    stop_event=threading.Event(),
                    watch_module=sys.modules["__main__"],
                )
                server = Server(
                    config=config, reloader=reloader, watchfn=watchfn_spaces
                )
            elif GRADIO_WATCH_DIRS:
                reloader = SourceFileReloader(
                    app=app,
                    watch_dirs=GRADIO_WATCH_DIRS,
                    watch_module_name=GRADIO_WATCH_MODULE_NAME,
                    demo_name=GRADIO_WATCH_DEMO_NAME,
                    stop_event=threading.Event(),
                    demo_file=GRADIO_WATCH_DEMO_PATH,
                    watch_module=sys.modules["__main__"],
                    encoding=os.getenv("GRADIO_WATCH_ENCODING", "utf-8"),
                )
                server = Server(config=config, reloader=reloader)
            else:
                server = Server(config=config)
            server.run_in_thread()
            break
        except (OSError, ServerFailedToStartError):
            pass
    else:
        raise OSError(
            f"Cannot find empty port in range: {min(server_ports)}-{max(server_ports)}. You can specify a different port by setting the GRADIO_SERVER_PORT environment variable or passing the `server_port` parameter to `launch()`."
        )

    if ssl_keyfile is not None:
        path_to_local_server = f"https://{url_host_name}:{port}/"
    else:
        path_to_local_server = f"http://{url_host_name}:{port}/"

    return server_name, port, path_to_local_server, server
